@@ -4,7 +4,7 @@
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the Simplified BSD License (also
 ** known as the "2-Clause License" or "FreeBSD License".)
-
+**
 ** This program is distributed in the hope that it will be useful,
 ** but without any warranty; without even the implied warranty of
 ** merchantability or fitness for a particular purpose.
@@ -101,9 +101,11 @@ void content_cache_insert(int rid, Blob *pBlob){
 }
 
 /*
-** Clear the content cache.
+** Clear the content cache. If it is passed true, it
+** also frees all associated memory, otherwise it may
+** retain parts for future uses of the cache.
 */
-void content_clear_cache(void){
+void content_clear_cache(int bFreeIt){
   int i;
   for(i=0; i<contentCache.n; i++){
     blob_reset(&contentCache.a[i].content);
@@ -113,6 +115,11 @@ void content_clear_cache(void){
   bag_clear(&contentCache.inCache);
   contentCache.n = 0;
   contentCache.szTotal = 0;
+  if(bFreeIt){
+    fossil_free(contentCache.a);
+    contentCache.a = 0;
+    contentCache.nAlloc = 0;
+  }
 }
 
 /*
@@ -600,7 +607,7 @@ int content_put_ex(
     }
   }
   if( g.markPrivate || isPrivate ){
-    db_multi_exec("INSERT INTO private VALUES(%d)", rid);
+    db_multi_exec("INSERT OR IGNORE INTO private VALUES(%d)", rid);
     markAsUnclustered = 0;
   }
   if( nBlob==0 ) blob_reset(&cmpr);
@@ -707,7 +714,7 @@ void test_content_put_cmd(void){
   if( g.argc!=3 ) usage("FILENAME");
   db_must_be_within_tree();
   user_select();
-  blob_read_from_file(&content, g.argv[2]);
+  blob_read_from_file(&content, g.argv[2], ExtFILE);
   rid = content_put(&content);
   fossil_print("inserted as record %d\n", rid);
 }
@@ -773,6 +780,18 @@ void content_make_public(int rid){
   db_exec(&s1);
 }
 
+/*
+** Make sure an artifact is private
+*/
+void content_make_private(int rid){
+  static Stmt s1;
+  db_static_prepare(&s1,
+    "INSERT OR IGNORE INTO private(rid) VALUES(:rid)"
+  );
+  db_bind_int(&s1, ":rid", rid);
+  db_exec(&s1);
+}
+
 /*
 ** Try to change the storage of rid so that it is a delta from one
 ** of the artifacts given in aSrc[0]..aSrc[nSrc-1].  The aSrc[*] that
@@ -928,8 +947,14 @@ static int looks_like_control_artifact(Blob *p){
 **
 ** Options:
 **
+**    -d|--db-only       Run "PRAGMA integrity_check" on the database only.
+**                       No other validation is performed.
+**
 **    --parse            Parse all manifests, wikis, tickets, events, and
 **                       so forth, reporting any errors found.
+**
+**    -q|--quick         Run "PRAGMA quick_check" on the database only.
+**                       No other validation is performed.
 */
 void test_integrity(void){
   Stmt q;
@@ -941,7 +966,21 @@ void test_integrity(void){
   int nCA = 0;
   int anCA[10];
   int bParse = find_option("parse",0,0)!=0;
+  int bDbOnly = find_option("db-only","d",0)!=0;
+  int bQuick = find_option("quick","q",0)!=0;
   db_find_and_open_repository(OPEN_ANY_SCHEMA, 2);
+  if( bDbOnly || bQuick ){
+    const char *zType = bQuick ? "quick" : "integrity";
+    char *zRes;
+    zRes = db_text(0,"PRAGMA repository.%s_check", zType/*safe-for-%s*/);
+    if( fossil_strcmp(zRes,"ok")!=0 ){
+      fossil_print("%s_check failed!\n", zType);
+      exit(1);
+    }else{
+      fossil_print("ok\n");
+    }
+    return;
+  }
   memset(anCA, 0, sizeof(anCA));
 
   /* Make sure no public artifact is a delta from a private artifact */
@@ -1220,7 +1259,7 @@ void test_missing(void){
 ** the metadata.
 **
 ** Note that the arguments are the integer raw RID values from the BLOB table,
-** not artifact hashs or labels.
+** not artifact hashes or labels.
 */
 void test_content_erase(void){
   int i;
