@@ -41,7 +41,7 @@ const char *mimetype_from_content(Blob *pBlob){
   */
   static const struct {
     const char *zPrefix;       /* The file prefix */
-    int size;                  /* Length of the prefix */
+    const int size;            /* Length of the prefix */
     const char *zMimetype;     /* The corresponding mimetype */
   } aMime[] = {
     { "GIF87a",                  6, "image/gif"  },
@@ -271,6 +271,7 @@ static const struct {
   { "vrml",       4, "model/vrml"                        },
   { "wav",        3, "audio/x-wav"                       },
   { "wax",        3, "audio/x-ms-wax"                    },
+  { "webp",       4, "image/webp"                        },
   { "wiki",       4, "text/x-fossil-wiki"                },
   { "wma",        3, "audio/x-ms-wma"                    },
   { "wmv",        3, "video/x-ms-wmv"                    },
@@ -300,12 +301,105 @@ static void mimetype_verify(void){
   int i;
   for(i=1; i<count(aMime); i++){
     if( fossil_strcmp(aMime[i-1].zSuffix,aMime[i].zSuffix)>=0 ){
-      fossil_fatal("mimetypes out of sequence: %s before %s",
+      fossil_panic("mimetypes out of sequence: %s before %s",
                    aMime[i-1].zSuffix, aMime[i].zSuffix);
     }
   }
 }
 
+/*
+** Looks in the contents of the "mimetypes" setting for a suffix
+** matching zSuffix. If found, it returns the configured value
+** in memory owned by the app (i.e. do not free() it), else it
+** returns 0.
+**
+** The mimetypes setting is expected to be a list of file extensions
+** and mimetypes, with one such mapping per line. A leading '.'  on
+** extensions is permitted for compatibility with lists imported from
+** other tools which require them.
+*/
+static const char *mimetype_from_name_custom(const char *zSuffix){
+  static char * zList = 0;
+  static char const * zEnd = 0;
+  static int once = 0;
+  char * z;
+  int tokenizerState /* 0=expecting a key, 1=skip next token,
+                     ** 2=accept next token */;
+  if(once==0){
+    once = 1; 
+    zList = db_get("mimetypes",0);
+    if(zList==0){
+      return 0;
+    }
+    /* Transform zList to simplify the main loop:
+       replace non-newline spaces with NUL bytes. */
+    zEnd = zList + strlen(zList);
+    for(z = zList; z<zEnd; ++z){
+      if('\n'==*z) continue;
+      else if(fossil_isspace(*z)){
+        *z = 0;
+      }
+    }
+  }else if(zList==0){
+    return 0;
+  }
+  tokenizerState = 0;
+  z = zList;
+  while( z<zEnd ){
+    if(*z==0){
+      ++z;
+      continue;
+    }
+    else if('\n'==*z){
+      if(2==tokenizerState){
+        /* We were expecting a value for a successful match
+           here, but got no value. Bail out. */
+        break;
+      }else{
+        /* May happen on malformed inputs. Skip this record. */
+        tokenizerState = 0;
+        ++z;
+        continue;
+      }
+    }
+    switch(tokenizerState){
+      case 0:{ /* This is a file extension */
+        static char * zCase = 0;
+        if('.'==*z){
+          /*ignore an optional leading dot, for compatibility
+            with some external mimetype lists*/;
+          if(++z==zEnd){
+            break;
+          }
+        }
+        if(zCase<z){
+          /*we have not yet case-folded this section: lower-case it*/
+          for(zCase = z; zCase<zEnd && *zCase!=0; ++zCase){
+            if(!(0x80 & *zCase)){
+              *zCase = (char)fossil_tolower(*zCase);
+            }
+          }
+        }
+        if(strcmp(z,zSuffix)==0){
+          tokenizerState = 2 /* Match: accept the next value. */;
+        }else{
+          tokenizerState = 1 /* No match: skip the next value. */;
+        }
+        z += strlen(z);
+        break;
+      }
+      case 1: /* This is a value, but not a match. Skip it. */
+        z += strlen(z);
+        break;
+      case 2: /* This is the value which matched the previous key. */;
+        return z;
+      default:
+        assert(!"cannot happen - invalid tokenizerState value.");
+    }
+  }
+  return 0;
+}
+
 /*
 ** Guess the mime-type of a document based on its name.
 */
@@ -335,6 +429,10 @@ const char *mimetype_from_name(const char *zName){
   if( len<sizeof(zSuffix)-1 ){
     sqlite3_snprintf(sizeof(zSuffix), zSuffix, "%s", z);
     for(i=0; zSuffix[i]; i++) zSuffix[i] = fossil_tolower(zSuffix[i]);
+    z = mimetype_from_name_custom(zSuffix);
+    if(z!=0){
+      return z;
+    }
     first = 0;
     last = count(aMime) - 1;
     while( first<=last ){
@@ -366,6 +464,7 @@ const char *mimetype_from_name(const char *zName){
 void mimetype_test_cmd(void){
   int i;
   mimetype_verify();
+  db_find_and_open_repository(0, 0);
   for(i=2; i<g.argc; i++){
     fossil_print("%-20s -> %s\n", g.argv[i], mimetype_from_name(g.argv[i]));
   }
@@ -379,11 +478,50 @@ void mimetype_test_cmd(void){
 */
 void mimetype_list_page(void){
   int i;
+  char *zCustomList = 0;    /* value of the mimetypes setting */
+  int nCustomEntries = 0;   /* number of entries in the mimetypes
+                            ** setting */
   mimetype_verify();
   style_header("Mimetype List");
   @ <p>The Fossil <a href="%R/help?cmd=/doc">/doc</a> page uses filename
-  @ suffixes and the following table to guess at the appropriate mimetype
-  @ for each document.</p>
+  @ suffixes and the following tables to guess at the appropriate mimetype
+  @ for each document. Mimetypes may be customized and overridden using
+  @ <a href="%R/help?cmd=mimetypes">the mimetypes config setting</a>.</p>
+  zCustomList = db_get("mimetypes",0);
+  if( zCustomList!=0 ){
+    Blob list, entry, key, val;
+    @ <h1>Repository-specific mimetypes</h1>
+    @ <p>The following extension-to-mimetype mappings are defined via
+    @ the <a href="%R/help?cmd=mimetypes">mimetypes setting</a>.</p>
+    @ <table class='sortable mimetypetable' border=1 cellpadding=0 \
+    @ data-column-types='tt' data-init-sort='0'>
+    @ <thead>
+    @ <tr><th>Suffix<th>Mimetype
+    @ </thead>
+    @ <tbody>
+    blob_set(&list, zCustomList);
+    while( blob_line(&list, &entry)>0 ){
+      const char *zKey;
+      if( blob_token(&entry, &key)==0 ) continue;
+      if( blob_token(&entry, &val)==0 ) continue;
+      zKey = blob_str(&key);
+      if( zKey[0]=='.' ) zKey++;
+      @ <tr><td>%h(zKey)<td>%h(blob_str(&val))</tr>
+      nCustomEntries++;
+    }
+    fossil_free(zCustomList);
+    if( nCustomEntries==0 ){
+      /* This can happen if the option is set to an empty/space-only
+      ** value. */
+      @ <tr><td colspan="2"><em>none</em></tr>
+    }
+    @ </tbody></table>
+  }
+  @ <h1>Default built-in mimetypes</h1>
+  if(nCustomEntries>0){
+    @ <p>Entries starting with an exclamation mark <em><strong>!</strong></em>
+    @ are overwritten by repository-specific settings.</p>
+  }
   @ <table class='sortable mimetypetable' border=1 cellpadding=0 \
   @ data-column-types='tt' data-init-sort='1'>
   @ <thead>
@@ -391,7 +529,12 @@ void mimetype_list_page(void){
   @ </thead>
   @ <tbody>
   for(i=0; i<count(aMime); i++){
-    @ <tr><td>%h(aMime[i].zSuffix)<td>%h(aMime[i].zMimetype)</tr>
+    const char *zFlag = "";
+    if(nCustomEntries>0 &&
+       mimetype_from_name_custom(aMime[i].zSuffix)!=0){
+      zFlag = "<em><strong>!</strong></em> ";
+    }
+    @ <tr><td>%s(zFlag)%h(aMime[i].zSuffix)<td>%h(aMime[i].zMimetype)</tr>
   }
   @ </tbody></table>
   style_table_sorter();
@@ -424,7 +567,7 @@ int doc_is_embedded_html(Blob *pContent, Blob *pTitle){
   zIn += 4;
   while( zIn[0] ){
     if( fossil_isspace(zIn[0]) ) zIn++;
-    if( zIn[0]=='>' ) return 0;
+    if( zIn[0]=='>' ) break;
     zAttr = zIn;
     while( fossil_isalnum(zIn[0]) || zIn[0]=='-' ) zIn++;
     nAttr = (int)(zIn - zAttr);
@@ -453,7 +596,14 @@ int doc_is_embedded_html(Blob *pContent, Blob *pTitle){
       if( seenTitle ) return 1;
     }
     if( nAttr==10 && fossil_strnicmp(zAttr,"data-title",10)==0 ){
-      blob_append(pTitle, zValue, nValue);
+      /* The text argument to data-title="" will have had any characters that
+      ** are special to HTML encoded.  We need to decode these before turning
+      ** the text into a title, as the title text will be reencoded later */
+      char *zTitle = mprintf("%.*s", nValue, zValue);
+      int i;
+      for(i=0; fossil_isspace(zTitle[i]); i++){}
+      html_to_plaintext(zTitle+i, pTitle);
+      fossil_free(zTitle);
       seenTitle = 1;
       if( seenClass ) return 1;
     }
@@ -469,6 +619,10 @@ int doc_is_embedded_html(Blob *pContent, Blob *pTitle){
 int doc_load_content(int vid, const char *zName, Blob *pContent){
   int writable = db_is_writeable("repository");
   int rid;   /* The RID of the file being loaded */
+  if( writable ){
+    db_end_transaction(0);
+    db_begin_write();
+  }
   if( !db_table_exists("repository", "vcache") || !writable ){
     db_multi_exec(
       "CREATE %s TABLE IF NOT EXISTS vcache(\n"
@@ -498,17 +652,51 @@ int doc_load_content(int vid, const char *zName, Blob *pContent){
   return rid;
 }
 
+/*
+** Check to verify that z[i] is contained within HTML markup.
+**
+** This works by looking backwards in the string for the most recent
+** '<' or '>' character.  If a '<' is found first, then we assume that
+** z[i] is within markup.  If a '>' is seen or neither character is seen,
+** then z[i] is not within markup.
+*/
+static int isWithinHtmlMarkup(const char *z, int i){
+  while( i>=0 && z[i]!='>' && z[i]!='<' ){ i--; }
+  return z[i]=='<';
+}
+
+/*
+** Check to see if z[i] is contained within an href='...' of markup.
+*/
+static int isWithinHref(const char *z, int i){
+  while( i>5
+     && !fossil_isspace(z[i])
+     && z[i]!='\'' && z[i]!='"'
+     && z[i]!='>'
+  ){ i--; }
+  if( i<=6 ) return 0;
+  if( z[i]!='\'' && z[i]!='\"' ) return 0;
+  if( strncmp(&z[i-5],"href=",5)!=0 ) return 0;
+  if( !fossil_isspace(z[i-6]) ) return 0;
+  return 1;
+}
+
 /*
 ** Transfer content to the output.  During the transfer, when text of
 ** the following form is seen:
 **
-**       href="$ROOT/
-**       action="$ROOT/
+**       href="$ROOT/..."
+**       action="$ROOT/..."
+**       href=".../doc/$CURRENT/..."
 **
-** Convert $ROOT to the root URI of the repository.  Allow ' in place of "
-** and any case for href or action.
+** Convert $ROOT to the root URI of the repository, and $CURRENT to the 
+** version number of the /doc/ document currently being displayed (if any).
+** Allow ' in place of " and any case for href or action.  
+**
+** Efforts are made to limit this translation to cases where the text is
+** fully contained with an HTML markup element.
 */
-static void convert_href_and_output(Blob *pIn){
+void convert_href_and_output(Blob *pIn){
   int i, base;
   int n = blob_size(pIn);
   char *z = blob_buffer(pIn);
@@ -517,17 +705,106 @@ static void convert_href_and_output(Blob *pIn){
      && strncmp(&z[i],"$ROOT/", 6)==0
      && (z[i-1]=='\'' || z[i-1]=='"')
      && i-base>=9
-     && (fossil_strnicmp(&z[i-7]," href=", 6)==0 ||
-           fossil_strnicmp(&z[i-9]," action=", 8)==0)
+     && ((fossil_strnicmp(&z[i-6],"href=",5)==0 && fossil_isspace(z[i-7])) ||
+         (fossil_strnicmp(&z[i-8],"action=",7)==0 && fossil_isspace(z[i-9])) )
+     && isWithinHtmlMarkup(z, i-6)
     ){
       blob_append(cgi_output_blob(), &z[base], i-base);
       blob_appendf(cgi_output_blob(), "%R");
       base = i+5;
+    }else
+    if( z[i]=='$'
+     && strncmp(&z[i-5],"/doc/$CURRENT/", 11)==0
+     && isWithinHref(z,i-5)
+     && isWithinHtmlMarkup(z, i-5)
+     && strncmp(g.zPath, "doc/",4)==0
+    ){
+      int j;
+      for(j=7; g.zPath[j] && g.zPath[j]!='/'; j++){}
+      blob_append(cgi_output_blob(), &z[base], i-base);
+      blob_appendf(cgi_output_blob(), "%.*s", j-4, g.zPath+4);
+      base = i+8;
     }
   }
   blob_append(cgi_output_blob(), &z[base], i-base);
 }
 
+/*
+** Render a document as the reply to the HTTP request.  The body
+** of the document is contained in pBody.  The body might be binary.
+** The mimetype is in zMimetype.
+*/
+void document_render(
+  Blob *pBody,                  /* Document content */
+  const char *zMime,            /* MIME-type */
+  const char *zDefaultTitle,    /* Default title */
+  const char *zFilename         /* Name of the file being rendered */
+){
+  Blob title;
+  blob_init(&title,0,0);
+  if( fossil_strcmp(zMime, "text/x-fossil-wiki")==0 ){
+    Blob tail;
+    style_adunit_config(ADUNIT_RIGHT_OK);
+    if( wiki_find_title(pBody, &title, &tail) ){
+      style_header("%s", blob_str(&title));
+      wiki_convert(&tail, 0, WIKI_BUTTONS);
+    }else{
+      style_header("%s", zDefaultTitle);
+      wiki_convert(pBody, 0, WIKI_BUTTONS);
+    }
+    style_footer();
+  }else if( fossil_strcmp(zMime, "text/x-markdown")==0 ){
+    Blob tail = BLOB_INITIALIZER;
+    markdown_to_html(pBody, &title, &tail);
+    if( blob_size(&title)>0 ){
+      style_header("%s", blob_str(&title));
+    }else{
+      style_header("%s", zDefaultTitle);
+    }
+    convert_href_and_output(&tail);
+    style_footer();
+  }else if( fossil_strcmp(zMime, "text/plain")==0 ){
+    style_header("%s", zDefaultTitle);
+    @ <blockquote><pre>
+    @ %h(blob_str(pBody))
+    @ </pre></blockquote>
+    style_footer();
+  }else if( fossil_strcmp(zMime, "text/html")==0
+            && doc_is_embedded_html(pBody, &title) ){
+    if( blob_size(&title)==0 ) blob_append(&title,zFilename,-1);
+    style_header("%s", blob_str(&title));
+    convert_href_and_output(pBody);
+    style_footer();
+#ifdef FOSSIL_ENABLE_TH1_DOCS
+  }else if( Th_AreDocsEnabled() &&
+            fossil_strcmp(zMime, "application/x-th1")==0 ){
+    int raw = P("raw")!=0;
+    if( !raw ){
+      Blob tail;
+      blob_zero(&tail);
+      if( wiki_find_title(pBody, &title, &tail) ){
+        style_header("%s", blob_str(&title));
+        Th_Render(blob_str(&tail));
+        blob_reset(&tail);
+      }else{
+        style_header("%h", zFilename);
+        Th_Render(blob_str(pBody));
+      }
+    }else{
+      Th_Render(blob_str(pBody));
+    }
+    if( !raw ){
+      style_footer();
+    }
+#endif
+  }else{
+    fossil_free(style_csp(1));
+    cgi_set_content_type(zMime);
+    cgi_set_content(pBody);
+  }
+}
+
+
 /*
 ** WEBPAGE: uv
 ** WEBPAGE: doc
@@ -535,7 +812,7 @@ static void convert_href_and_output(Blob *pIn){
 ** URL: /doc/CHECKIN/FILE
 **
 ** CHECKIN can be either tag or hash prefix or timestamp identifying a
-** particular check, or the name of a branch (meaning the most recent
+** particular check-in, or the name of a branch (meaning the most recent
 ** check-in on that branch) or one of various magic words:
 **
 **     "tip"      means the most recent check-in
@@ -543,10 +820,14 @@ static void convert_href_and_output(Blob *pIn){
 **     "ckout"    means the current check-out, if the server is run from
 **                within a check-out, otherwise it is the same as "tip"
 **
+**     "latest"   means use the most recent check-in for the document
+**                regardless of what branch it occurs on.
+**
 ** FILE is the name of a file to delivered up as a webpage.  FILE is relative
 ** to the root of the source tree of the repository. The FILE must
 ** be a part of CHECKIN, except when CHECKIN=="ckout" when FILE is read
-** directly from disk and need not be a managed file.
+** directly from disk and need not be a managed file.  For /uv, FILE
+** can also be the hash of the unversioned file.
 **
 ** The "ckout" CHECKIN is intended for development - to provide a mechanism
 ** for looking at what a file will look like using the /doc webpage after
@@ -612,10 +893,21 @@ void doc_page(void){
       zCheckin = mprintf("%.*s", i, zName);
       if( fossil_strcmp(zCheckin,"ckout")==0 && g.localOpen==0 ){
         zCheckin = "tip";
+      }else if( fossil_strcmp(zCheckin,"latest")==0 ){
+        char *zNewCkin = db_text(0,
+          "SELECT uuid FROM blob, mlink, event, filename"
+          " WHERE filename.name=%Q"
+          "   AND mlink.fnid=filename.fnid"
+          "   AND blob.rid=mlink.mid"
+          "   AND event.objid=mlink.mid"
+          " ORDER BY event.mtime DESC LIMIT 1",
+          zName + i + 1);
+        if( zNewCkin ) zCheckin = zNewCkin;
       }
     }
     if( nMiss==count(azSuffix) ){
       zName = "404.md";
+      zDfltTitle = "Not Found";
     }else if( zName[i]==0 ){
       assert( nMiss>=0 && nMiss<count(azSuffix) );
       zName = azSuffix[nMiss];
@@ -642,18 +934,22 @@ void doc_page(void){
     }
     if( isUV ){
       if( db_table_exists("repository","unversioned") ){
-        Stmt q;
-        db_prepare(&q, "SELECT hash, mtime FROM unversioned"
-                       " WHERE name=%Q", zName);
-        if( db_step(&q)==SQLITE_ROW ){
-          etag_check(ETAG_HASH, db_column_text(&q,0));
-          etag_last_modified(db_column_int64(&q,1));
-        }
-        db_finalize(&q);
-        if( unversioned_content(zName, &filebody)==0 ){
-          rid = 1;
-          zDfltTitle = zName;
+        rid = unversioned_content(zName, &filebody);
+        if( rid==1 ){
+          Stmt q;
+          db_prepare(&q, "SELECT hash, mtime FROM unversioned"
+                         " WHERE name=%Q", zName);
+          if( db_step(&q)==SQLITE_ROW ){
+            etag_check(ETAG_HASH, db_column_text(&q,0));
+            etag_last_modified(db_column_int64(&q,1));
+          }
+          db_finalize(&q);
+        }else if( rid==2 ){
+          zName = db_text(zName,
+             "SELECT name FROM unversioned WHERE hash=%Q", zName);
+          g.isConst = 1;
         }
+        zDfltTitle = zName;
       }
     }else if( fossil_strcmp(zCheckin,"ckout")==0 ){
       /* Read from the local checkout */
@@ -666,8 +962,8 @@ void doc_page(void){
       }
       fossil_free(zFullpath);
     }else{
-      vid = name_to_typed_rid(zCheckin, "ci");
-      rid = doc_load_content(vid, zName, &filebody);
+      vid = symbolic_name_to_rid(zCheckin, "ci");
+      rid = vid>0 ? doc_load_content(vid, zName, &filebody) : 0;
     }
   }
   g.zPath = mprintf("%s/%s", g.zPath, zPathSuffix);
@@ -688,66 +984,7 @@ void doc_page(void){
     Th_Store("doc_date", db_text(0, "SELECT datetime(mtime) FROM event"
                                     " WHERE objid=%d AND type='ci'", vid));
   }
-  if( fossil_strcmp(zMime, "text/x-fossil-wiki")==0 ){
-    Blob tail;
-    style_adunit_config(ADUNIT_RIGHT_OK);
-    if( wiki_find_title(&filebody, &title, &tail) ){
-      style_header("%s", blob_str(&title));
-      wiki_convert(&tail, 0, WIKI_BUTTONS);
-    }else{
-      style_header("%s", zDfltTitle);
-      wiki_convert(&filebody, 0, WIKI_BUTTONS);
-    }
-    style_footer();
-  }else if( fossil_strcmp(zMime, "text/x-markdown")==0 ){
-    Blob tail = BLOB_INITIALIZER;
-    markdown_to_html(&filebody, &title, &tail);
-    if( blob_size(&title)>0 ){
-      style_header("%s", blob_str(&title));
-    }else{
-      style_header("%s", nMiss>=count(azSuffix)?
-                        "Not Found" : zDfltTitle);
-    }
-    convert_href_and_output(&tail);
-    style_footer();
-  }else if( fossil_strcmp(zMime, "text/plain")==0 ){
-    style_header("%s", zDfltTitle);
-    @ <blockquote><pre>
-    @ %h(blob_str(&filebody))
-    @ </pre></blockquote>
-    style_footer();
-  }else if( fossil_strcmp(zMime, "text/html")==0
-            && doc_is_embedded_html(&filebody, &title) ){
-    if( blob_size(&title)==0 ) blob_append(&title,zName,-1);
-    style_header("%s", blob_str(&title));
-    convert_href_and_output(&filebody);
-    style_footer();
-#ifdef FOSSIL_ENABLE_TH1_DOCS
-  }else if( Th_AreDocsEnabled() &&
-            fossil_strcmp(zMime, "application/x-th1")==0 ){
-    int raw = P("raw")!=0;
-    if( !raw ){
-      Blob tail;
-      blob_zero(&tail);
-      if( wiki_find_title(&filebody, &title, &tail) ){
-        style_header("%s", blob_str(&title));
-        Th_Render(blob_str(&tail));
-        blob_reset(&tail);
-      }else{
-        style_header("%h", zName);
-        Th_Render(blob_str(&filebody));
-      }
-    }else{
-      Th_Render(blob_str(&filebody));
-    }
-    if( !raw ){
-      style_footer();
-    }
-#endif
-  }else{
-    cgi_set_content_type(zMime);
-    cgi_set_content(&filebody);
-  }
+  document_render(&filebody, zMime, zDfltTitle, zName);
   if( nMiss>=count(azSuffix) ) cgi_set_status(404, "Not Found");
   db_end_transaction(0);
   return;
@@ -766,7 +1003,6 @@ doc_not_found:
     @ in %z(href("%R/tree?ci=%T",zCheckin))%h(zCheckin)</a>
   }
   style_footer();
-  db_end_transaction(0);
   return;
 }
 
@@ -895,6 +1131,30 @@ void background_page(void){
 }
 
 
+/*
+** WEBPAGE: favicon.ico
+**
+** Return the default favicon.ico image.  The returned image is for the
+** Fossil lizard icon.
+**
+** The intended use case here is to supply a favicon for the "fossil ui"
+** command.  For a permanent website, the recommended process is for
+** the admin to set up a project-specific favicon and reference that
+** icon in the HTML header using a line like:
+**
+**   <link rel="icon" href="URL-FOR-YOUR-ICON" type="MIMETYPE"/>
+** 
+*/
+void favicon_page(void){
+  Blob favicon;
+
+  etag_check(ETAG_CONFIG, 0);
+  blob_zero(&favicon);
+  blob_init(&favicon, (char*)aLogo, sizeof(aLogo));
+  cgi_set_content_type("image/gif");
+  cgi_set_content(&favicon);
+}
+
 /*
 ** WEBPAGE: docsrch
 **
