@@ -201,7 +201,7 @@ void finfo_cmd(void){
     );
     blob_zero(&line);
     if( iBrief ){
-      fossil_print("History of %s\n", blob_str(&fname));
+      fossil_print("History for %s\n", blob_str(&fname));
     }
     while( db_step(&q)==SQLITE_ROW ){
       const char *zFileUuid = db_column_text(&q, 0);
@@ -298,7 +298,7 @@ void cat_cmd(void){
 */
 void finfo_page(void){
   Stmt q;
-  const char *zFilename;
+  const char *zFilename = PD("name","");
   char zPrevDate[20];
   const char *zA;
   const char *zB;
@@ -323,7 +323,12 @@ void finfo_page(void){
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
-  style_header("File History");
+  fnid = db_int(0, "SELECT fnid FROM filename WHERE name=%Q", zFilename);
+  if( fnid==0 ){
+    style_header("No such file");
+  }else{
+    style_header("History for %s", zFilename);
+  }
   login_anonymous_available();
   tmFlags = timeline_ss_submenu();
   if( tmFlags & TIMELINE_COLUMNAR ){
@@ -342,9 +347,7 @@ void finfo_page(void){
   if( uBg ) url_add_parameter(&url, "ubg", 0);
   baseCheckin = name_to_rid_www("ci");
   zPrevDate[0] = 0;
-  zFilename = PD("name","");
   cookie_render();
-  fnid = db_int(0, "SELECT fnid FROM filename WHERE name=%Q", zFilename);
   if( fnid==0 ){
     @ No such file: %h(zFilename)
     style_footer();
@@ -437,8 +440,9 @@ void finfo_page(void){
     }else{
       blob_appendf(&title, "Ancestors of file ");
     }
-    blob_appendf(&title,"<a href='%R/finfo?name=%T'>%h</a>",
-                 zFilename, zFilename);
+    blob_appendf(&title,"%z%h</a>",
+                 href("%R/file?name=%T&ci=%!S", zFilename, zUuid),
+                 zFilename);
     if( fShowId ) blob_appendf(&title, " (%d)", fnid);
     blob_append(&title, origCheckin ? " between " : " from ", -1);
     blob_appendf(&title, "check-in %z%S</a>", zLink, zUuid);
@@ -451,10 +455,13 @@ void finfo_page(void){
       fossil_free(zUuid);
     }
   }else{
-    blob_appendf(&title, "History of ");
-    hyperlinked_path(zFilename, &title, 0, "tree", "");
+    blob_appendf(&title, "History for ");
+    hyperlinked_path(zFilename, &title, 0, "tree", "", LINKPATH_FILE);
     if( fShowId ) blob_appendf(&title, " (%d)", fnid);
   }
+  if( uBg ){
+    blob_append(&title, " (color-coded by user)", -1);
+  }
   @ <h2>%b(&title)</h2>
   blob_reset(&title);
   pGraph = graph_init();
@@ -523,7 +530,7 @@ void finfo_page(void){
       @ <tr>
     }
     @ <td class="timelineTime">\
-    @ %z(href("%R/artifact/%!S",zUuid))%s(zTime)</a></td>
+    @ %z(href("%R/file?name=%T&ci=%!S",zFilename,zCkin))%s(zTime)</a></td>
     @ <td class="timelineGraph"><div id="m%d(gidx)" class="tl-nodemark"></div>
     @ </td>
     if( zBgClr && zBgClr[0] ){
@@ -560,7 +567,7 @@ void finfo_page(void){
     cgi_printf("<span class='timeline%sDetail'>", zStyle);
     if( tmFlags & (TIMELINE_COMPACT|TIMELINE_VERBOSE) ) cgi_printf("(");
     if( zUuid && (tmFlags & TIMELINE_VERBOSE)==0 ){
-      @ file:&nbsp;%z(href("%R/artifact/%!S",zUuid))[%S(zUuid)]</a>
+      @ file:&nbsp;%z(href("%R/file?name=%T&ci=%!S",zFilename,zCkin))[%S(zUuid)]</a>
       if( fShowId ){
         int srcId = delta_source_rid(frid);
         if( srcId>0 ){
@@ -782,7 +789,7 @@ void mlink_page(void){
        mid
     );
     @ <h1>MLINK table for check-in %h(zCI)</h1>
-    render_checkin_context(mid, 1);
+    render_checkin_context(mid, 0, 1);
     style_table_sorter();
     @ <hr />
     @ <div class='brlist'>
