@@ -36,6 +36,7 @@
 #define CFTYPE_TICKET     5
 #define CFTYPE_ATTACHMENT 6
 #define CFTYPE_EVENT      7
+#define CFTYPE_FORUM      8
 
 /*
 ** File permissions used by Fossil internally.
@@ -69,7 +70,7 @@ struct Manifest {
   Blob content;         /* The original content blob */
   int type;             /* Type of artifact.  One of CFTYPE_xxxxx */
   int rid;              /* The blob-id for this manifest */
-  char *zBaseline;      /* Baseline manifest.  The B card. */
+  const char *zBaseline;/* Baseline manifest.  The B card. */
   Manifest *pBaseline;  /* The actual baseline manifest */
   char *zComment;       /* Decoded comment.  The C card. */
   double rDate;         /* Date and time from D card.  0.0 if no D card. */
@@ -78,12 +79,15 @@ struct Manifest {
   char *zWiki;          /* Text of the wiki page.  W card. */
   char *zWikiTitle;     /* Name of the wiki page. L card. */
   char *zMimetype;      /* Mime type of wiki or comment text.  N card.  */
+  char *zThreadTitle;   /* The forum thread title. H card */
   double rEventDate;    /* Date of an event.  E card. */
   char *zEventId;       /* Artifact hash for an event.  E card. */
   char *zTicketUuid;    /* UUID for a ticket. K card. */
   char *zAttachName;    /* Filename of an attachment. A card. */
   char *zAttachSrc;     /* Artifact hash for document being attached. A card. */
   char *zAttachTarget;  /* Ticket or wiki that attachment applies to.  A card */
+  char *zThreadRoot;    /* Thread root artifact.  G card */
+  char *zInReplyTo;     /* Forum in-reply-to artifact.  I card */
   int nFile;            /* Number of F cards */
   int nFileAlloc;       /* Slots allocated in aFile[] */
   int iFile;            /* Index of current file in iterator */
@@ -115,6 +119,40 @@ struct Manifest {
 };
 #endif
 
+/*
+** Allowed and required card types in each style of artifact
+*/
+static struct {
+  const char *zAllowed;     /* Allowed cards.  Human-readable */
+  const char *zRequired;    /* Required cards.  Human-readable */
+} manifestCardTypes[] = {
+  /*                           Allowed          Required    */
+  /* CFTYPE_MANIFEST   1 */ { "BCDFNPQRTUZ",   "DZ"          },
+                     /* Wants to be "CDUZ" ----^^^^
+                     ** but we must limit for historical compatibility */
+  /* CFTYPE_CLUSTER    2 */ { "MZ",            "MZ"          },
+  /* CFTYPE_CONTROL    3 */ { "DTUZ",          "DTUZ"        },
+  /* CFTYPE_WIKI       4 */ { "CDLNPUWZ",      "DLUWZ"       },
+  /* CFTYPE_TICKET     5 */ { "DJKUZ",         "DJKUZ"       },
+  /* CFTYPE_ATTACHMENT 6 */ { "ACDNUZ",        "ADZ"         },
+  /* CFTYPE_EVENT      7 */ { "CDENPTUWZ",     "DEWZ"        },
+  /* CFTYPE_FORUM      8 */ { "DGHINPUWZ",     "DUWZ"        },
+};
+
+/*
+** Names of manifest types
+*/
+static const char *const azNameOfMType[] = {
+  "manifest",
+  "cluster",
+  "tag",
+  "wiki",
+  "ticket",
+  "attachment",
+  "technote",
+  "forum post"
+};
+
 /*
 ** A cache of parsed manifests.  This reduces the number of
 ** calls to manifest_parse() when doing a rebuild.
@@ -150,6 +188,31 @@ void manifest_destroy(Manifest *p){
   }
 }
 
+/*
+** Given a string of upper-case letters, compute a mask of the letters
+** present.  For example,  "ABC" computes 0x0007.  "DE" gives 0x0018".
+*/
+static unsigned int manifest_card_mask(const char *z){
+  unsigned int m = 0;
+  char c;
+  while( (c = *(z++))>='A' && c<='Z' ){
+    m |= 1 << (c - 'A');
+  }
+  return m;
+}
+
+/*
+** Given an integer mask representing letters A-Z, return the
+** letter which is the first bit set in the mask.  Example:
+** 0x03520 gives 'F' since the F-bit is the lowest.
+*/
+static char maskToType(unsigned int x){
+  char c = 'A';
+  if( x==0 ) return '?';
+  while( (x&1)==0 ){ x >>= 1; c++; }
+  return c;
+}
+
 /*
 ** Add an element to the manifest cache using LRU replacement.
 */
@@ -257,14 +320,17 @@ static void remove_pgp_signature(char **pz, int *pn){
 **   0123456789 123456789 123456789 123456789
 **   Z aea84f4f863865a8d59d0384e4d2a41c
 */
-static int verify_z_card(const char *z, int n){
+static int verify_z_card(const char *z, int n, Blob *pErr){
+  const char *zHash;
   if( n<35 ) return 0;
   if( z[n-35]!='Z' || z[n-34]!=' ' ) return 0;
   md5sum_init();
   md5sum_step_text(z, n-35);
-  if( memcmp(&z[n-33], md5sum_finish(0), 32)==0 ){
+  zHash = md5sum_finish(0);
+  if( memcmp(&z[n-33], zHash, 32)==0 ){
     return 1;
   }else{
+    blob_appendf(pErr, "incorrect Z-card cksum: expected %.32s", zHash);
     return 2;
   }
 }
@@ -284,16 +350,15 @@ struct ManifestText {
 ** Return NULL if there are no more tokens on the current line.
 */
 static char *next_token(ManifestText *p, int *pLen){
-  char *z;
   char *zStart;
-  int c;
+  int n;
   if( p->atEol ) return 0;
-  zStart = z = p->z;
-  while( (c=(*z))!=' ' && c!='\n' ){ z++; }
-  *z = 0;
-  p->z = &z[1];
-  p->atEol = c=='\n';
-  if( pLen ) *pLen = z - zStart;
+  zStart = p->z;
+  n = strcspn(p->z, " \n");
+  p->atEol = p->z[n]=='\n';
+  p->z[n] = 0;
+  p->z += n+1;
+  if( pLen ) *pLen = n;
   return zStart;
 }
 
@@ -322,6 +387,19 @@ static char next_card(ManifestText *p){
 */
 #define SYNTAX(T)  {zErr=(T); goto manifest_syntax_error;}
 
+/*
+** A cache of manifest IDs which manifest_parse() has seen in this
+** session.
+*/
+static Bag seenManifests =  Bag_INIT;
+/*
+** Frees all memory owned by the manifest "has-seen" cache.  Intended
+** to be called only from the app's atexit() handler.
+*/
+void manifest_clear_cache(){
+  bag_clear(&seenManifests);
+}
+
 /*
 ** Parse a blob into a Manifest object.  The Manifest object
 ** takes over the input blob and will free it when the
@@ -354,7 +432,6 @@ static char next_card(ManifestText *p){
 */
 Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   Manifest *p;
-  int seenZ = 0;
   int i, lineNo=0;
   ManifestText x;
   char cPrevType = 0;
@@ -363,17 +440,21 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   int n;
   char *zUuid;
   int sz = 0;
-  int isRepeat, hasSelfRefTag = 0;
-  static Bag seen;
+  int isRepeat;
+  int nSelfTag = 0;     /* Number of T cards referring to this manifest */
+  int nSimpleTag = 0;   /* Number of T cards with "+" prefix */
   const char *zErr = 0;
+  unsigned int m;
+  unsigned int seenCard = 0;   /* Which card types have been seen */
+  char zErrBuf[100];           /* Write error messages here */
 
   if( rid==0 ){
     isRepeat = 1;
-  }else if( bag_find(&seen, rid) ){
+  }else if( bag_find(&seenManifests, rid) ){
     isRepeat = 1;
   }else{
     isRepeat = 0;
-    bag_insert(&seen, rid);
+    bag_insert(&seenManifests, rid);
   }
 
   /* Every structural artifact ends with a '\n' character.  Exit early
@@ -402,9 +483,8 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   }
   /* Then verify the Z-card.
   */
-  if( verify_z_card(z, n)==2 ){
+  if( verify_z_card(z, n, pErr)==2 ){
     blob_reset(pContent);
-    blob_appendf(pErr, "incorrect Z-card cksum");
     return 0;
   }
 
@@ -424,6 +504,8 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   x.atEol = 1;
   while( (cType = next_card(&x))!=0 && cType>=cPrevType ){
     lineNo++;
+    if( cType<'A' || cType>'Z' ) SYNTAX("bad card type");
+    seenCard |= 1 << (cType-'A');
     switch( cType ){
       /*
       **     A <filename> <target> ?<source>?
@@ -442,7 +524,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( zName==0 || zTarget==0 ) goto manifest_syntax_error;
         if( p->zAttachName!=0 ) goto manifest_syntax_error;
         defossilize(zName);
-        if( !file_is_simple_pathname(zName, 0) ){
+        if( !file_is_simple_pathname_nonstrict(zName) ){
           SYNTAX("invalid filename on A-card");
         }
         defossilize(zTarget);
@@ -456,6 +538,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         p->zAttachName = (char*)file_tail(zName);
         p->zAttachSrc = zSrc;
         p->zAttachTarget = zTarget;
+        p->type = CFTYPE_ATTACHMENT;
         break;
       }
 
@@ -471,6 +554,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( !hname_validate(p->zBaseline,sz) ){
           SYNTAX("invalid hash on B-card");
         }
+        p->type = CFTYPE_MANIFEST;
         break;
       }
 
@@ -522,6 +606,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( !hname_validate(p->zEventId, sz) ){
           SYNTAX("malformed hash on E-card");
         }
+        p->type = CFTYPE_EVENT;
         break;
       }
 
@@ -537,7 +622,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         zName = next_token(&x,0);
         if( zName==0 ) SYNTAX("missing filename on F-card");
         defossilize(zName);
-        if( !file_is_simple_pathname(zName, 0) ){
+        if( !file_is_simple_pathname_nonstrict(zName) ){
           SYNTAX("F-card filename is not a simple path");
         }
         zUuid = next_token(&x, &sz);
@@ -550,7 +635,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         zPriorName = next_token(&x,0);
         if( zPriorName ){
           defossilize(zPriorName);
-          if( !file_is_simple_pathname(zPriorName, 0) ){
+          if( !file_is_simple_pathname_nonstrict(zPriorName) ){
             SYNTAX("F-card old filename is not a simple path");
           }
         }
@@ -567,6 +652,55 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( i>0 && fossil_strcmp(p->aFile[i-1].zName, zName)>=0 ){
           SYNTAX("incorrect F-card sort order");
         }
+        p->type = CFTYPE_MANIFEST;
+        break;
+      }
+
+      /*
+      **    G <hash>
+      **
+      ** A G-card identifies the initial root forum post for the thread
+      ** of which this post is a part.  Forum posts only.
+      */
+      case 'G': {
+        if( p->zThreadRoot!=0 ) SYNTAX("more than one G-card");
+        p->zThreadRoot = next_token(&x, &sz);
+        if( p->zThreadRoot==0 ) SYNTAX("missing hash on G-card");
+        if( !hname_validate(p->zThreadRoot,sz) ){
+          SYNTAX("Invalid hash on G-card");
+        }
+        p->type = CFTYPE_FORUM;
+        break;
+      }
+
+      /*
+      **     H <threadtitle>
+      **
+      ** The title for a forum thread.
+      */
+      case 'H': {
+        if( p->zThreadTitle!=0 ) SYNTAX("more than one H-card");
+        p->zThreadTitle = next_token(&x,0);
+        if( p->zThreadTitle==0 ) SYNTAX("missing title on H-card");
+        defossilize(p->zThreadTitle);
+        p->type = CFTYPE_FORUM;
+        break;
+      }
+
+      /*
+      **    I <hash>
+      **
+      ** A I-card identifies another forum post that the current forum post
+      ** is in reply to.
+      */
+      case 'I': {
+        if( p->zInReplyTo!=0 ) SYNTAX("more than one I-card");
+        p->zInReplyTo = next_token(&x, &sz);
+        if( p->zInReplyTo==0 ) SYNTAX("missing hash on I-card");
+        if( !hname_validate(p->zInReplyTo,sz) ){
+          SYNTAX("Invalid hash on I-card");
+        }
+        p->type = CFTYPE_FORUM;
         break;
       }
 
@@ -596,6 +730,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( i>0 && fossil_strcmp(p->aField[i-1].zName, zName)>=0 ){
           SYNTAX("incorrect J-card sort order");
         }
+        p->type = CFTYPE_TICKET;
         break;
       }
 
@@ -613,6 +748,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( !validate16(p->zTicketUuid, sz) ){
           SYNTAX("invalid K-card UUID");
         }
+        p->type = CFTYPE_TICKET;
         break;
       }
 
@@ -630,6 +766,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( !wiki_name_is_wellformed((const unsigned char *)p->zWikiTitle) ){
           SYNTAX("L-card has malformed wiki name");
         }
+        p->type = CFTYPE_WIKI;
         break;
       }
 
@@ -655,6 +792,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( i>0 && fossil_strcmp(p->azCChild[i-1], zUuid)>=0 ){
           SYNTAX("M-card in the wrong order");
         }
+        p->type = CFTYPE_CLUSTER;
         break;
       }
 
@@ -719,6 +857,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( zUuid && !hname_validate(zUuid,sz) ){
           SYNTAX("invalid second hash on Q-card");
         }
+        p->type = CFTYPE_MANIFEST;
         break;
       }
 
@@ -733,6 +872,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         p->zRepoCksum = next_token(&x, &sz);
         if( sz!=32 ) SYNTAX("wrong size cksum on R-card");
         if( !validate16(p->zRepoCksum, 32) ) SYNTAX("malformed R-card cksum");
+        p->type = CFTYPE_MANIFEST;
         break;
       }
 
@@ -761,13 +901,9 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( zValue ) defossilize(zValue);
         if( hname_validate(zUuid, sz) ){
           /* A valid artifact hash */
-          if( p->zEventId ) SYNTAX("non-self-referential T-card in event");
         }else if( sz==1 && zUuid[0]=='*' ){
           zUuid = 0;
-          hasSelfRefTag = 1;
-          if( p->zEventId && zName[0]!='+' ){
-            SYNTAX("propagating T-card in event");
-          }
+          nSelfTag++;
         }else{
           SYNTAX("malformed artifact hash on T-card");
         }
@@ -775,6 +911,7 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         if( zName[0]!='-' && zName[0]!='+' && zName[0]!='*' ){
           SYNTAX("T-card name does not begin with '-', '+', or '*'");
         }
+        if( zName[0]=='+' ) nSimpleTag++;
         if( validate16(&zName[1], strlen(&zName[1])) ){
           /* Do not allow tags whose names look like a hash */
           SYNTAX("T-card name looks like a hexadecimal hash");
@@ -860,7 +997,6 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
         zUuid = next_token(&x, &sz);
         if( sz!=32 ) SYNTAX("wrong size for Z-card cksum");
         if( !validate16(zUuid, 32) ) SYNTAX("malformed Z-card cksum");
-        seenZ = 1;
         break;
       }
       default: {
@@ -870,81 +1006,60 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   }
   if( x.z<x.zEnd ) SYNTAX("extra characters at end of card");
 
-  if( p->nCChild>0 ){
-    if( p->zAttachName
-     || p->zBaseline
-     || p->zComment
-     || p->rDate>0.0
-     || p->zEventId
-     || p->nFile>0
-     || p->nField>0
-     || p->zTicketUuid
-     || p->zWikiTitle
-     || p->zMimetype
-     || p->nParent>0
-     || p->nCherrypick>0
-     || p->zRepoCksum
-     || p->nTag>0
-     || p->zUser
-     || p->zWiki
-    ){
-      SYNTAX("cluster contains a card other than M- or Z-");
+  /* If the artifact type has not yet been determined, then compute
+  ** it now. */
+  if( p->type==0 ){
+    if( p->zComment!=0 || p->nFile>0 || p->nParent>0 ){
+      p->type = CFTYPE_MANIFEST;
+    }else{
+      p->type = CFTYPE_CONTROL;
     }
-    if( !seenZ ) SYNTAX("missing Z-card on cluster");
-    p->type = CFTYPE_CLUSTER;
-  }else if( p->zEventId ){
-    if( p->zAttachName ) SYNTAX("A-card in event");
-    if( p->zBaseline ) SYNTAX("B-card in event");
-    if( p->rDate<=0.0 ) SYNTAX("missing date on event");
-    if( p->nFile>0 ) SYNTAX("F-card in event");
-    if( p->nField>0 ) SYNTAX("J-card in event");
-    if( p->zTicketUuid ) SYNTAX("K-card in event");
-    if( p->zWikiTitle!=0 ) SYNTAX("L-card in event");
-    if( p->zRepoCksum ) SYNTAX("R-card in event");
-    if( p->zWiki==0 ) SYNTAX("missing W-card on event");
-    if( !seenZ ) SYNTAX("missing Z-card on event");
-    p->type = CFTYPE_EVENT;
-  }else if( p->zWiki!=0 || p->zWikiTitle!=0 ){
-    if( p->zAttachName ) SYNTAX("A-card in wiki");
-    if( p->zBaseline ) SYNTAX("B-card in wiki");
-    if( p->rDate<=0.0 ) SYNTAX("missing date on wiki");
-    if( p->nFile>0 ) SYNTAX("F-card in wiki");
-    if( p->nField>0 ) SYNTAX("J-card in wiki");
-    if( p->zTicketUuid ) SYNTAX("K-card in wiki");
-    if( p->zWikiTitle==0 ) SYNTAX("missing L-card on wiki");
-    if( p->zRepoCksum ) SYNTAX("R-card in wiki");
-    if( p->nTag>0 ) SYNTAX("T-card in wiki");
-    if( p->zWiki==0 ) SYNTAX("missing W-card on wiki");
-    if( !seenZ ) SYNTAX("missing Z-card on wiki");
-    p->type = CFTYPE_WIKI;
-  }else if( hasSelfRefTag || p->nFile>0 || p->zRepoCksum!=0 || p->zBaseline
-      || p->nParent>0 ){
-    if( p->zAttachName ) SYNTAX("A-card in manifest");
-    if( p->rDate<=0.0 ) SYNTAX("missing date on manifest");
-    if( p->nField>0 ) SYNTAX("J-card in manifest");
-    if( p->zTicketUuid ) SYNTAX("K-card in manifest");
-    p->type = CFTYPE_MANIFEST;
-  }else if( p->nField>0 || p->zTicketUuid!=0 ){
-    if( p->zAttachName ) SYNTAX("A-card in ticket");
-    if( p->rDate<=0.0 ) SYNTAX("missing date on ticket");
-    if( p->nField==0 ) SYNTAX("missing J-card on ticket");
-    if( p->zTicketUuid==0 ) SYNTAX("missing K-card on ticket");
-    if( p->zMimetype) SYNTAX("N-card in ticket");
-    if( p->nTag>0 ) SYNTAX("T-card in ticket");
-    if( p->zUser==0 ) SYNTAX("missing U-card on ticket");
-    if( !seenZ ) SYNTAX("missing Z-card on ticket");
-    p->type = CFTYPE_TICKET;
-  }else if( p->zAttachName ){
-    if( p->rDate<=0.0 ) SYNTAX("missing date on attachment");
-    if( p->nTag>0 ) SYNTAX("T-card in attachment");
-    if( !seenZ ) SYNTAX("missing Z-card on attachment");
-    p->type = CFTYPE_ATTACHMENT;
-  }else{
-    if( p->rDate<=0.0 ) SYNTAX("missing date on control");
-    if( p->zMimetype ) SYNTAX("N-card in control");
-    if( !seenZ ) SYNTAX("missing Z-card on control");
-    p->type = CFTYPE_CONTROL;
   }
+
+  /* Verify that no disallowed cards are present for this artifact type */
+  m = manifest_card_mask(manifestCardTypes[p->type-1].zAllowed);
+  if( seenCard & ~m ){
+    sqlite3_snprintf(sizeof(zErrBuf), zErrBuf, "%c-card in %s",
+                     maskToType(seenCard & ~m),
+                     azNameOfMType[p->type-1]);
+    zErr = zErrBuf;
+    goto manifest_syntax_error;
+  }
+
+  /* Verify that all required cards are present for this artifact type */
+  m = manifest_card_mask(manifestCardTypes[p->type-1].zRequired);
+  if( ~seenCard & m ){
+    sqlite3_snprintf(sizeof(zErrBuf), zErrBuf, "%c-card missing in %s",
+                     maskToType(~seenCard & m),
+                     azNameOfMType[p->type-1]);
+    zErr = zErrBuf;
+    goto manifest_syntax_error;
+  }
+
+  /* Additional checks based on artifact type */
+  switch( p->type ){
+    case CFTYPE_CONTROL: {
+      if( nSelfTag ) SYNTAX("self-referential T-card in control artifact");
+      break;
+    }
+    case CFTYPE_EVENT: {
+      if( p->nTag!=nSelfTag ){
+        SYNTAX("non-self-referential T-card in technote");
+      }
+      if( p->nTag!=nSimpleTag ){
+        SYNTAX("T-card with '*' or '-' in technote");
+      }
+      break;
+    }
+    case CFTYPE_FORUM: {
+      if( p->zThreadTitle && p->zInReplyTo ){
+        SYNTAX("cannot have I-card and H-card in a forum post");
+      }
+      if( p->nParent>1 ) SYNTAX("too many arguments to P-card");
+      break;
+    }
+  }
+
   md5sum_init();
   if( !isRepeat ) g.parseCnt[p->type]++;
   return p;
@@ -953,7 +1068,7 @@ manifest_syntax_error:
   {
     char *zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
     if( zUuid ){
-      blob_appendf(pErr, "manifest [%s] ", zUuid);
+      blob_appendf(pErr, "artifact [%s] ", zUuid);
       fossil_free(zUuid);
     }
   }
@@ -1017,14 +1132,16 @@ Manifest *manifest_get_by_name(const char *zName, int *pRid){
 **
 ** Usage: %fossil test-parse-manifest FILENAME ?N?
 **
-** Parse the manifest and discarded.  Use for testing only.
+** Parse the manifest(s) given on the command-line and report any
+** errors.  If the N argument is given, run the parsing N times.
 */
 void manifest_test_parse_cmd(void){
   Manifest *p;
   Blob b;
   int i;
   int n = 1;
-  sqlite3_open(":memory:", &g.db);
+  db_find_and_open_repository(0,0);
+  verify_all_options();
   if( g.argc!=3 && g.argc!=4 ){
     usage("FILENAME");
   }
@@ -1040,6 +1157,50 @@ void manifest_test_parse_cmd(void){
     blob_reset(&err);
     manifest_destroy(p);
   }
+  blob_reset(&b);
+}
+
+/*
+** COMMAND: test-parse-all-blobs
+**
+** Usage: %fossil test-parse-all-blobs [--limit N]
+**
+** Parse all entries in the BLOB table that are believed to be non-data
+** artifacts and report any errors.  Run this test command on historical
+** repositories after making any changes to the manifest_parse()
+** implementation to confirm that the changes did not break anything.
+**
+** If the --limit N argument is given, parse no more than N blobs
+*/
+void manifest_test_parse_all_blobs_cmd(void){
+  Manifest *p;
+  Blob err;
+  Stmt q;
+  int nTest = 0;
+  int nErr = 0;
+  int N = 1000000000;
+  const char *z;
+  db_find_and_open_repository(0, 0);
+  z = find_option("limit", 0, 1);
+  if( z ) N = atoi(z);
+  verify_all_options();
+  db_prepare(&q, "SELECT DISTINCT objid FROM EVENT");
+  while( (N--)>0 && db_step(&q)==SQLITE_ROW ){
+    int id = db_column_int(&q,0);
+    fossil_print("Checking %d       \r", id);
+    nTest++;
+    fflush(stdout);
+    blob_init(&err, 0, 0);
+    p = manifest_get(id, CFTYPE_ANY, &err);
+    if( p==0 ){
+      fossil_print("%d ERROR: %s\n", id, blob_str(&err));
+      nErr++;
+    }
+    blob_reset(&err);
+    manifest_destroy(p);
+  }
+  db_finalize(&q);
+  fossil_print("%d tests with %d errors\n", nTest, nErr);
 }
 
 /*
@@ -1557,7 +1718,7 @@ static int manifest_add_checkin_linkages(
   int rid,                   /* The RID of the check-in */
   Manifest *p,               /* Manifest for this check-in */
   int nParent,               /* Number of parents for this check-in */
-  char **azParent            /* hashes for each parent */
+  char * const * azParent    /* hashes for each parent */
 ){
   int i;
   int parentid = 0;
@@ -1669,7 +1830,7 @@ void manifest_crosslink_begin(void){
   manifest_crosslink_busy = 1;
   db_begin_transaction();
   db_multi_exec(
-     "CREATE TEMP TABLE pending_tkt(uuid TEXT UNIQUE);"
+     "CREATE TEMP TABLE pending_xlink(id TEXT PRIMARY KEY)WITHOUT ROWID;"
      "CREATE TEMP TABLE time_fudge("
      "  mid INTEGER PRIMARY KEY,"    /* The rid of a manifest */
      "  m1 REAL,"                    /* The timestamp on mid */
@@ -1679,6 +1840,17 @@ void manifest_crosslink_begin(void){
   );
 }
 
+/*
+** Add a new entry to the pending_xlink table.
+*/
+static void add_pending_crosslink(char cType, const char *zId){
+  assert( manifest_crosslink_busy==1 );
+  db_multi_exec(
+    "INSERT OR IGNORE INTO pending_xlink VALUES('%c%q')",
+    cType, zId
+  );
+}
+
 #if INTERFACE
 /* Timestamps might be adjusted slightly to ensure that check-ins appear
 ** on the timeline in chronological order.  This is the maximum amount
@@ -1720,16 +1892,24 @@ int manifest_crosslink_end(int flags){
     manifest_reparent_checkin(rid, zValue);
   }
   db_finalize(&q);
-  db_prepare(&q, "SELECT uuid FROM pending_tkt");
+  db_prepare(&q, "SELECT id FROM pending_xlink");
   while( db_step(&q)==SQLITE_ROW ){
-    const char *zUuid = db_column_text(&q, 0);
-    ticket_rebuild_entry(zUuid);
-    if( permitHooks && rc==TH_OK ){
-      rc = xfer_run_script(zScript, zUuid, 0);
+    const char *zId = db_column_text(&q, 0);
+    char cType;
+    if( zId==0 || zId[0]==0 ) continue;
+    cType = zId[0];
+    zId++;
+    if( cType=='t' ){
+      ticket_rebuild_entry(zId);
+      if( permitHooks && rc==TH_OK ){
+        rc = xfer_run_script(zScript, zId, 0);
+      }
+    }else if( cType=='w' ){
+      backlink_wiki_refresh(zId);
     }
   }
   db_finalize(&q);
-  db_multi_exec("DROP TABLE pending_tkt");
+  db_multi_exec("DROP TABLE pending_xlink");
 
   /* If multiple check-ins happen close together in time, adjust their
   ** times by a few milliseconds to make sure they appear in chronological
@@ -1756,7 +1936,8 @@ int manifest_crosslink_end(int flags){
   if( db_exists("SELECT 1 FROM time_fudge") ){
     db_multi_exec(
       "UPDATE event SET mtime=(SELECT m1 FROM time_fudge WHERE mid=objid)"
-      " WHERE objid IN (SELECT mid FROM time_fudge);"
+      " WHERE objid IN (SELECT mid FROM time_fudge)"
+      " AND (mtime=omtime OR omtime IS NULL)"
     );
   }
   db_multi_exec("DROP TABLE time_fudge;");
@@ -1899,7 +2080,7 @@ static int tag_compare(const void *a, const void *b){
 
 /*
 ** Scan artifact rid/pContent to see if it is a control artifact of
-** any key:
+** any type:
 **
 **      *  Manifest
 **      *  Control
@@ -1908,6 +2089,7 @@ static int tag_compare(const void *a, const void *b){
 **      *  Cluster
 **      *  Attachment
 **      *  Event
+**      *  Forum post
 **
 ** If the input is a control artifact, then make appropriate entries
 ** in the auxiliary tables of the database in order to crosslink the
@@ -1931,13 +2113,17 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
   const char *zScript = 0;
   const char *zUuid = 0;
 
+  if( g.fSqlTrace ){
+    fossil_trace("-- manifest_crosslink(%d)\n", rid);
+  }
   if( (p = manifest_cache_find(rid))!=0 ){
     blob_reset(pContent);
   }else if( (p = manifest_parse(pContent, rid, 0))==0 ){
     assert( blob_is_reset(pContent) || pContent==0 );
     if( (flags & MC_NO_ERRORS)==0 ){
-      fossil_error(1, "syntax error in manifest [%S]",
-                   db_text(0, "SELECT uuid FROM blob WHERE rid=%d",rid));
+      char * zErrUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d",rid);
+      fossil_error(1, "syntax error in manifest [%S]", zErrUuid);
+      fossil_free(zErrUuid);
     }
     return 0;
   }
@@ -1962,6 +2148,17 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
       zScript = xfer_commit_code();
       zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
     }
+    if( p->nCherrypick && db_table_exists("repository","cherrypick") ){
+      int i;
+      for(i=0; i<p->nCherrypick; i++){
+        db_multi_exec(
+          "REPLACE INTO cherrypick(parentid,childid,isExclude)"
+          " SELECT rid, %d, %d FROM blob WHERE uuid=%Q",
+          rid, p->aCherrypick[i].zCPTarget[0]=='-',
+          p->aCherrypick[i].zCPTarget+1
+        );
+      }
+    }
     if( !db_exists("SELECT 1 FROM mlink WHERE mid=%d", rid) ){
       char *zCom;
       parentid = manifest_add_checkin_linkages(rid,p,p->nParent,p->azParent);
@@ -1986,7 +2183,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
       );
       zCom = db_text(0, "SELECT coalesce(ecomment, comment) FROM event"
                         " WHERE rowid=last_insert_rowid()");
-      wiki_extract_links(zCom, rid, 0, p->rDate, 1, WIKI_INLINE);
+      backlink_extract(zCom, 0, rid, BKLNK_COMMENT, p->rDate, 1);
       fossil_free(zCom);
 
       /* If this is a delta-manifest, record the fact that this repository
@@ -2035,6 +2232,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
           case '*':  type = 2;  break;  /* Propagate to descendants */
           default:
             fossil_error(1, "unknown tag type in manifest: %s", p->aTag);
+            manifest_destroy(p);
             return 0;
         }
         tag_insert(&p->aTag[i].zName[1], type, p->aTag[i].zValue,
@@ -2050,6 +2248,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     int tagid = tag_findid(zTag, 1);
     int prior;
     char *zComment;
+    const char *zPrefix;
     int nWiki;
     char zLength[40];
     while( fossil_isspace(p->zWiki[0]) ) p->zWiki++;
@@ -2066,12 +2265,40 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     if( prior ){
       content_deltify(prior, &rid, 1, 0);
     }
-    if( nWiki>0 ){
-      zComment = mprintf("Changes to wiki page [%h]", p->zWikiTitle);
+    if( nWiki<=0 ){
+      zPrefix = "Deleted";
+    }else if( !prior ){
+      zPrefix = "Added";
     }else{
-      zComment = mprintf("Deleted wiki page [%h]", p->zWikiTitle);
+      zPrefix = "Changes to";
+    }
+    switch( wiki_page_type(p->zWikiTitle) ){
+      case WIKITYPE_CHECKIN: {
+        zComment = mprintf("%s wiki for check-in [%S]", zPrefix,
+                           p->zWikiTitle+8);
+        break;
+      }
+      case WIKITYPE_BRANCH: {
+        zComment = mprintf("%s wiki for branch [/timeline?r=%t|%h]",
+                           zPrefix, p->zWikiTitle+7, p->zWikiTitle+7);
+        break;
+      }
+      case WIKITYPE_TAG: {
+        zComment = mprintf("%s wiki for tag [/timeline?t=%t|%h]",
+                           zPrefix, p->zWikiTitle+4, p->zWikiTitle+4);
+        break;
+      }
+      default: {
+        zComment = mprintf("%s wiki page [%h]", zPrefix, p->zWikiTitle);
+        break;
+      }
     }
     search_doc_touch('w',rid,p->zWikiTitle);
+    if( manifest_crosslink_busy ){
+      add_pending_crosslink('w',p->zWikiTitle);
+    }else{
+      backlink_wiki_refresh(p->zWikiTitle);
+    }
     db_multi_exec(
       "REPLACE INTO event(type,mtime,objid,user,comment,"
       "                  bgcolor,euser,ecomment)"
@@ -2172,8 +2399,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     zTag = mprintf("tkt-%s", p->zTicketUuid);
     tag_insert(zTag, 1, 0, rid, p->rDate, rid);
     fossil_free(zTag);
-    db_multi_exec("INSERT OR IGNORE INTO pending_tkt VALUES(%Q)",
-                  p->zTicketUuid);
+    add_pending_crosslink('t',p->zTicketUuid);
     /* Locate and update comment for any attachments */
     db_prepare(&qatt,
        "SELECT attachid, src, target, filename FROM attachment"
@@ -2377,6 +2603,68 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     );
     blob_reset(&comment);
   }
+  if( p->type==CFTYPE_FORUM ){
+    int froot, fprev, firt;
+    char *zFType;
+    char *zTitle;
+    schema_forum();
+    search_doc_touch('f', rid, 0);
+    froot = p->zThreadRoot ? uuid_to_rid(p->zThreadRoot, 1) : rid;
+    fprev = p->nParent ? uuid_to_rid(p->azParent[0],1) : 0;
+    firt = p->zInReplyTo ? uuid_to_rid(p->zInReplyTo,1) : 0;
+    db_multi_exec(
+      "REPLACE INTO forumpost(fpid,froot,fprev,firt,fmtime)"
+      "VALUES(%d,%d,nullif(%d,0),nullif(%d,0),%.17g)",
+      p->rid, froot, fprev, firt, p->rDate
+    );
+    if( firt==0 ){
+      /* This is the start of a new thread, either the initial entry
+      ** or an edit of the initial entry. */
+      zTitle = p->zThreadTitle;
+      if( zTitle==0 || zTitle[0]==0 ){
+        zTitle = "(Deleted)";
+      }
+      zFType = fprev ? "Edit" : "Post";
+      db_multi_exec(
+        "REPLACE INTO event(type,mtime,objid,user,comment)"
+        "VALUES('f',%.17g,%d,%Q,'%q: %q')",
+        p->rDate, rid, p->zUser, zFType, zTitle
+      );
+      /*
+      ** If this edit is the most recent, then make it the title for
+      ** all other entries for the same thread
+      */
+      if( !db_exists("SELECT 1 FROM forumpost WHERE froot=%d AND firt=0"
+                     "   AND fpid!=%d AND fmtime>%.17g", froot, rid, p->rDate)
+      ){
+        /* This entry establishes a new title for all entries on the thread */
+        db_multi_exec(
+          "UPDATE event"
+          " SET comment=substr(comment,1,instr(comment,':')) || ' %q'"
+          " WHERE objid IN (SELECT fpid FROM forumpost WHERE froot=%d)",
+          zTitle, froot
+        );
+      }
+    }else{
+      /* This is a reply to a prior post.  Take the title from the root. */
+      zTitle = db_text(0, "SELECT substr(comment,instr(comment,':')+2)"
+                          "  FROM event WHERE objid=%d", froot);
+      if( zTitle==0 ) zTitle = fossil_strdup("<i>Unknown</i>");
+      if( p->zWiki[0]==0 ){
+        zFType = "Delete reply";
+      }else if( fprev ){
+        zFType = "Edit reply";
+      }else{
+        zFType = "Reply";
+      }
+      db_multi_exec(
+        "REPLACE INTO event(type,mtime,objid,user,comment)"
+        "VALUES('f',%.17g,%d,%Q,'%q: %q')",
+        p->rDate, rid, p->zUser, zFType, zTitle
+      );
+      fossil_free(zTitle);
+    }
+  }
   db_end_transaction(0);
   if( permitHooks ){
     rc = xfer_run_common_script();
