@@ -36,7 +36,7 @@
 ** be overridden using the hash-digits setting.  FOSSIL_HASH_DIGITS_URL
 ** is the minimum number of digits to be used in URLs.  The number used
 ** will always be at least 6 more than the number used for human output,
-** or 40 if the number of digits in human output is 34 or more.
+** or HNAME_MAX, whichever is least.
 */
 #ifndef FOSSIL_HASH_DIGITS
 # define FOSSIL_HASH_DIGITS 10       /* For %S (human display) */
@@ -50,16 +50,16 @@
 ** human output if the bForUrl is false and is destined for a URL if
 ** bForUrl is false.
 */
-static int hashDigits(int bForUrl){
+int hash_digits(int bForUrl){
   static int nDigitHuman = 0;
   static int nDigitUrl = 0;
   if( nDigitHuman==0 ){
     nDigitHuman = db_get_int("hash-digits", FOSSIL_HASH_DIGITS);
     if( nDigitHuman < 6 ) nDigitHuman = 6;
-    if( nDigitHuman > 40 ) nDigitHuman = 40;
+    if( nDigitHuman > HNAME_MAX ) nDigitHuman = HNAME_MAX;
     nDigitUrl = nDigitHuman + 6;
     if( nDigitUrl < FOSSIL_HASH_DIGITS_URL ) nDigitUrl = FOSSIL_HASH_DIGITS_URL;
-    if( nDigitUrl > 40 ) nDigitUrl = 40;
+    if( nDigitUrl > HNAME_MAX ) nDigitUrl = HNAME_MAX;
   }
   return bForUrl ? nDigitUrl : nDigitHuman;
 }
@@ -68,7 +68,7 @@ static int hashDigits(int bForUrl){
 ** Return the number of characters in a %S output.
 */
 int length_of_S_display(void){
-  return hashDigits(0);
+  return hash_digits(0);
 }
 
 /*
@@ -101,6 +101,7 @@ int length_of_S_display(void){
 #define etWIKISTR    22 /* Timeline comment text rendered from a char*: %W */
 #define etSTRINGID   23 /* String with length limit for a UUID prefix: %S */
 #define etROOT       24 /* String value of g.zTop: %R */
+#define etJSONSTR    25 /* String encoded as a JSON string literal: %j */
 
 
 /*
@@ -152,6 +153,7 @@ static const et_info fmtinfo[] = {
   {  'w',  0, 4, etSQLESCAPE3, 0,  0 },
   {  'F',  0, 4, etFOSSILIZE,  0,  0 },
   {  'S',  0, 4, etSTRINGID,   0,  0 },
+  {  'j',  0, 0, etJSONSTR,    0,  0 },
   {  'c',  0, 0, etCHARX,      0,  0 },
   {  'o',  8, 0, etRADIX,      0,  2 },
   {  'u', 10, 0, etRADIX,      0,  0 },
@@ -204,11 +206,15 @@ static int et_getdigit(long double *val, int *cnt){
 ** N bytes then return N.  If N is negative, then this routine
 ** is an alias for strlen().
 */
+#if _XOPEN_SOURCE >= 700 || _POSIX_C_SOURCE >= 200809L
+# define StrNLen32(Z,N) (int)strnlen(Z,N)
+#else
 static int StrNLen32(const char *z, int N){
   int n = 0;
   while( (N-- != 0) && *(z++)!=0 ){ n++; }
   return n;
 }
+#endif
 
 /*
 ** Return an appropriate set of flags for wiki_convert() for displaying
@@ -231,6 +237,9 @@ static int wiki_convert_flags(int altForm2){
     if( db_get_boolean("timeline-plaintext", 0) ){
       wikiFlags |= WIKI_LINKSONLY;
     }
+    if( db_get_boolean("timeline-hard-newlines", 0) ){
+      wikiFlags |= WIKI_NEWLINE;
+    }
   }
   return wikiFlags;
 }
@@ -277,6 +286,7 @@ int vxprintf(
   etByte flag_long;          /* True if "l" flag is present */
   etByte flag_longlong;      /* True if the "ll" flag is present */
   etByte done;               /* Loop termination flag */
+  etByte cThousand;          /* Thousands separator for %d and %u */
   u64 longvalue;             /* Value for integer types */
   long double realvalue;     /* Value for real types */
   const et_info *infop;      /* Pointer to the appropriate info structure */
@@ -314,7 +324,7 @@ int vxprintf(
       break;
     }
     /* Find out what flags are present */
-    flag_leftjustify = flag_plussign = flag_blanksign =
+    flag_leftjustify = flag_plussign = flag_blanksign = cThousand =
      flag_alternateform = flag_altform2 = flag_zeropad = 0;
     done = 0;
     do{
@@ -325,6 +335,7 @@ int vxprintf(
         case '#':   flag_alternateform = 1;   break;
         case '!':   flag_altform2 = 1;        break;
         case '0':   flag_zeropad = 1;         break;
+        case ',':   cThousand = ',';          break;
         default:    done = 1;                 break;
       }
     }while( !done && (c=(*++fmt))!=0 );
@@ -456,8 +467,23 @@ int vxprintf(
           }while( longvalue>0 );
         }
         length = &buf[etBUFSIZE-1]-bufpt;
-        for(idx=precision-length; idx>0; idx--){
+        while( precision>length ){
           *(--bufpt) = '0';                             /* Zero pad */
+          length++;
+        }
+        if( cThousand ){
+          int nn = (length - 1)/3;  /* Number of "," to insert */
+          int ix = (length - 1)%3 + 1;
+          bufpt -= nn;
+          for(idx=0; nn>0; idx++){
+            bufpt[idx] = bufpt[idx+nn];
+            ix--;
+            if( ix==0 ){
+              bufpt[++idx] = cThousand;
+              nn--;
+              ix = 3;
+            }
+          }
         }
         if( prefix ) *(--bufpt) = prefix;               /* Add sign */
         if( flag_alternateform && infop->prefix ){      /* Add "0" or "0x" */
@@ -659,7 +685,7 @@ int vxprintf(
         }else if( xtype==etDYNSTRING ){
           zExtra = bufpt;
         }else if( xtype==etSTRINGID ){
-          precision = hashDigits(flag_altform2);
+          precision = hash_digits(flag_altform2);
         }
         length = StrNLen32(bufpt, limit);
         if( precision>=0 && precision<length ) length = precision;
@@ -765,6 +791,20 @@ int vxprintf(
         if( precision>=0 && precision<length ) length = precision;
         break;
       }
+      case etJSONSTR: {
+        int limit = flag_alternateform ? va_arg(ap,int) : -1;
+        char *zMem = va_arg(ap,char*);
+        if( limit!=0 ){
+          /* Ignore the limit flag, if set, for JSON string
+          ** output. This block exists to squelch the associated
+          ** "unused variable" compiler warning. */
+        }
+        if( zMem==0 ) zMem = "";
+        zExtra = bufpt = encode_json_string_literal(zMem);
+        length = strlen(bufpt);
+        if( precision>=0 && precision<length ) length = precision;
+        break;
+      }
       case etWIKISTR: {
         int limit = flag_alternateform ? va_arg(ap,int) : -1;
         char *zWiki = va_arg(ap, char*);
@@ -947,6 +987,16 @@ void fossil_print(const char *zFormat, ...){
   }
   va_end(ap);
 }
+void fossil_vprint(const char *zFormat, va_list ap){
+  if( g.cgiOutput ){
+    cgi_vprintf(zFormat, ap);
+  }else{
+    Blob b = empty_blob;
+    vxprintf(&b, zFormat, ap);
+    fossil_puts(blob_str(&b), 0);
+    blob_reset(&b);
+  }
+}
 
 /*
 ** Print a trace message on standard error.
@@ -966,23 +1016,28 @@ void fossil_trace(const char *zFormat, ...){
 ** Write a message to the error log, if the error log filename is
 ** defined.
 */
-static void fossil_errorlog(const char *zFormat, ...){
+void fossil_errorlog(const char *zFormat, ...){
   struct tm *pNow;
   time_t now;
   FILE *out;
   const char *z;
   int i;
   va_list ap;
-  static const char *const azEnv[] = { "HTTP_HOST", "HTTP_USER_AGENT",
+  static const char *const azEnv[] = { "HTTP_HOST", "HTTP_REFERER",
+      "HTTP_USER_AGENT",
       "PATH_INFO", "QUERY_STRING", "REMOTE_ADDR", "REQUEST_METHOD",
       "REQUEST_URI", "SCRIPT_NAME" };
   if( g.zErrlog==0 ) return;
-  out = fossil_fopen(g.zErrlog, "a");
-  if( out==0 ) return;
+  if( g.zErrlog[0]=='-' && g.zErrlog[1]==0 ){
+    out = stderr;
+  }else{
+    out = fossil_fopen(g.zErrlog, "a");
+    if( out==0 ) return;
+  }
   now = time(0);
   pNow = gmtime(&now);
   fprintf(out, "------------- %04d-%02d-%02d %02d:%02d:%02d UTC ------------\n",
-          pNow->tm_year+1900, pNow->tm_mon+1, pNow->tm_mday+1,
+          pNow->tm_year+1900, pNow->tm_mon+1, pNow->tm_mday,
           pNow->tm_hour, pNow->tm_min, pNow->tm_sec);
   va_start(ap, zFormat);
   vfprintf(out, zFormat, ap);
@@ -990,10 +1045,10 @@ static void fossil_errorlog(const char *zFormat, ...){
   va_end(ap);
   for(i=0; i<count(azEnv); i++){
     char *p;
-    if( (p = fossil_getenv(azEnv[i]))!=0 ){
+    if( (p = fossil_getenv(azEnv[i]))!=0 && p[0]!=0 ){
       fprintf(out, "%s=%s\n", azEnv[i], p);
       fossil_path_free(p);
-    }else if( (z = P(azEnv[i]))!=0 ){
+    }else if( (z = P(azEnv[i]))!=0 && z[0]!=0 ){
       fprintf(out, "%s=%s\n", azEnv[i], z);
     }
   }
@@ -1007,9 +1062,50 @@ static void fossil_errorlog(const char *zFormat, ...){
 */
 static int mainInFatalError = 0;
 
+/*
+** Write error message output
+*/
+static int fossil_print_error(int rc, const char *z){
+#ifdef FOSSIL_ENABLE_JSON
+  if( g.json.isJsonMode ){
+    json_err( 0, z, 1 );
+    if( g.isHTTP ){
+      rc = 0 /* avoid HTTP 500 */;
+    }
+  }
+  else
+#endif
+  if( g.cgiOutput==1 && g.db ){
+    g.cgiOutput = 2;
+    cgi_reset_content();
+    cgi_set_content_type("text/html");
+    style_header("Bad Request");
+    @ <p class="generalError">%h(z)</p>
+    cgi_set_status(400, "Bad Request");
+    style_footer();
+    cgi_reply();
+  }else if( !g.fQuiet ){
+    fossil_force_newline();
+    fossil_trace("%s\n", z);
+  }
+  return rc;
+}
+
 /*
 ** Print an error message, rollback all databases, and quit.  These
 ** routines never return.
+**
+** The only different between fossil_fatal() and fossil_panic() is that
+** fossil_panic() makes an entry in the error log whereas fossil_fatal()
+** does not.  If there is not error log, then both routines work the
+** same.  Hence, the routines are interchangable for commands and only
+** make a difference with processing web pages.
+**
+** Use fossil_fatal() for malformed inputs that should be reported back
+** to the user, but which do not represent a configuration problem or bug.
+**
+** Use fossil_panic() for any kind of error that should be brought to the
+** attention of the system administrator.
 */
 NORETURN void fossil_panic(const char *zFormat, ...){
   va_list ap;
@@ -1020,34 +1116,18 @@ NORETURN void fossil_panic(const char *zFormat, ...){
   if( once ) exit(1);
   once = 1;
   mainInFatalError = 1;
-  db_force_rollback();
+  /* db_force_rollback(); */
   va_start(ap, zFormat);
   sqlite3_vsnprintf(sizeof(z),z,zFormat, ap);
   va_end(ap);
+  if( g.fAnyTrace ){
+    fprintf(stderr, "/***** panic on %d *****/\n", getpid());
+  }
   fossil_errorlog("panic: %s", z);
-#ifdef FOSSIL_ENABLE_JSON
-  if( g.json.isJsonMode ){
-    json_err( 0, z, 1 );
-    if( g.isHTTP ){
-      rc = 0 /* avoid HTTP 500 */;
-    }
-  }
-  else
-#endif
-  {
-    if( g.cgiOutput ){
-      cgi_printf("<p class=\"generalError\">%h</p>", z);
-      cgi_reply();
-    }else if( !g.fQuiet ){
-      fossil_force_newline();
-      fossil_puts("Fossil internal error: ", 1);
-      fossil_puts(z, 1);
-      fossil_puts("\n", 1);
-    }
-  }
+  rc = fossil_print_error(rc, z);
+  abort();
   exit(rc);
 }
-
 NORETURN void fossil_fatal(const char *zFormat, ...){
   char *z;
   int rc = 1;
@@ -1056,27 +1136,8 @@ NORETURN void fossil_fatal(const char *zFormat, ...){
   va_start(ap, zFormat);
   z = vmprintf(zFormat, ap);
   va_end(ap);
-  fossil_errorlog("fatal: %s", z);
-#ifdef FOSSIL_ENABLE_JSON
-  if( g.json.isJsonMode ){
-    json_err( g.json.resultCode, z, 1 );
-    if( g.isHTTP ){
-      rc = 0 /* avoid HTTP 500 */;
-    }
-  }
-  else
-#endif
-  {
-    if( g.cgiOutput ){
-      g.cgiOutput = 0;
-      cgi_printf("<p class=\"generalError\">\n%h\n</p>\n", z);
-      cgi_reply();
-    }else if( !g.fQuiet ){
-      fossil_force_newline();
-      fossil_trace("%s\n", z);
-    }
-  }
-  free(z);
+  rc = fossil_print_error(rc, z);
+  fossil_free(z);
   db_force_rollback();
   fossil_exit(rc);
 }
@@ -1099,31 +1160,20 @@ void fossil_fatal_recursive(const char *zFormat, ...){
   va_start(ap, zFormat);
   z = vmprintf(zFormat, ap);
   va_end(ap);
-  fossil_errorlog("fatal: %s", z);
-#ifdef FOSSIL_ENABLE_JSON
-  if( g.json.isJsonMode ){
-    json_err( g.json.resultCode, z, 1 );
-    if( g.isHTTP ){
-      rc = 0 /* avoid HTTP 500 */;
-    }
-  } else
-#endif
-  {
-    if( g.cgiOutput ){
-      g.cgiOutput = 0;
-      cgi_printf("<p class=\"generalError\">\n%h\n</p>\n", z);
-      cgi_reply();
-    }else{
-      fossil_force_newline();
-      fossil_trace("%s\n", z);
-    }
-  }
+  rc = fossil_print_error(rc, z);
   db_force_rollback();
   fossil_exit(rc);
 }
 
 
-/* Print a warning message */
+/* Print a warning message.
+**
+** Unlike fossil_fatal() and fossil_panic(), this routine does return
+** and processing attempts to continue.  A message is written to the
+** error log, however, so this routine should only be used for situations
+** that require administrator or developer attention.  Minor problems
+** in user inputs should not use this routine.
+*/
 void fossil_warning(const char *zFormat, ...){
   char *z;
   va_list ap;
@@ -1133,11 +1183,11 @@ void fossil_warning(const char *zFormat, ...){
   fossil_errorlog("warning: %s", z);
 #ifdef FOSSIL_ENABLE_JSON
   if(g.json.isJsonMode){
-    json_warn( FSL_JSON_W_UNKNOWN, z );
+    json_warn( FSL_JSON_W_UNKNOWN, "%s", z );
   }else
 #endif
   {
-    if( g.cgiOutput ){
+    if( g.cgiOutput==1 ){
       cgi_printf("<p class=\"generalError\">\n%h\n</p>\n", z);
     }else{
       fossil_force_newline();
