@@ -127,6 +127,19 @@ static void sqlcmd_decompress(
   }
 }
 
+/*
+** Implementation of the "gather_artifact_stats(X)" SQL function.
+** That function merely calls the gather_artifact_stats() function
+** in stat.c to populate the ARTSTAT temporary table.
+*/
+static void sqlcmd_gather_artifact_stats(
+  sqlite3_context *context,
+  int argc,
+  sqlite3_value **argv
+){
+  gather_artifact_stats(1);
+}
+
 /*
 ** Add the content(), compress(), and decompress() SQL functions to
 ** database connection db.
@@ -138,6 +151,8 @@ int add_content_sql_commands(sqlite3 *db){
                           sqlcmd_compress, 0, 0);
   sqlite3_create_function(db, "decompress", 1, SQLITE_UTF8, 0,
                           sqlcmd_decompress, 0, 0);
+  sqlite3_create_function(db, "gather_artifact_stats", 0, SQLITE_UTF8, 0,
+                          sqlcmd_gather_artifact_stats, 0, 0);
   return SQLITE_OK;
 }
 
@@ -151,6 +166,7 @@ static int sqlcmd_autoinit(
   const char **pzErrMsg,
   const void *notUsed
 ){
+  int mTrace = SQLITE_TRACE_CLOSE;
   add_content_sql_commands(db);
   db_add_aux_functions(db);
   re_add_sql_func(db);
@@ -173,6 +189,10 @@ static int sqlcmd_autoinit(
     sqlite3_exec(db, zSql, 0, 0, 0);
     sqlite3_free(zSql);
   }
+  /* Arrange to trace close operations so that static prepared statements
+  ** will get cleaned up when the shell closes the database connection */
+  if( g.fSqlTrace ) mTrace |= SQLITE_TRACE_PROFILE;
+  sqlite3_trace_v2(db, mTrace, db_sql_trace, 0);
   return SQLITE_OK;
 }
 
@@ -304,7 +324,7 @@ static void fossil_close(int bDb, int noRepository){
 */
 void cmd_sqlite3(void){
   int noRepository;
-  const char *zConfigDb;
+  char *zConfigDb;
   extern int sqlite3_shell(int, char**);
 #ifdef FOSSIL_ENABLE_TH1_HOOKS
   g.fNoThHook = 1;
@@ -314,7 +334,7 @@ void cmd_sqlite3(void){
     db_find_and_open_repository(OPEN_ANY_SCHEMA, 0);
   }
   db_open_config(1,0);
-  zConfigDb = g.zConfigDbName;
+  zConfigDb = fossil_strdup(g.zConfigDbName);
   fossil_close(1, noRepository);
   sqlite3_shutdown();
 #ifndef _WIN32
