@@ -22,7 +22,6 @@
 #include "config.h"
 #include "style.h"
 
-
 /*
 ** Elements of the submenu are collected into the following
 ** structure and displayed below the main menu.
@@ -82,12 +81,24 @@ static int sideboxUsed = 0;
 */
 static unsigned adUnitFlags = 0;
 
+/*
+** Submenu disable flag
+*/
+static int submenuEnable = 1;
+
 /*
 ** Flags for various javascript files needed prior to </body>
 */
-static int needHrefJs = 0;   /* href.js */
-static int needSortJs = 0;   /* sorttable.js */
-static int needGraphJs = 0;  /* graph.js */
+static int needHrefJs = 0;      /* href.js */
+static int needSortJs = 0;      /* sorttable.js */
+static int needGraphJs = 0;     /* graph.js */
+static int needCopyBtnJs = 0;   /* copybtn.js */
+static int needAccordionJs = 0; /* accordion.js */
+
+/*
+** Extra JS added to the end of the file.
+*/
+static Blob blobOnLoad = BLOB_INITIALIZER;
 
 /*
 ** Generate and return a anchor tag like this:
@@ -139,13 +150,22 @@ char *xhref(const char *zExtra, const char *zFormat, ...){
   zUrl = vmprintf(zFormat, ap);
   va_end(ap);
   if( g.perm.Hyperlink && !g.javascriptHyperlink ){
-    char *zHUrl = mprintf("<a %s href=\"%h\">", zExtra, zUrl);
+    char *zHUrl;
+    if( zExtra ){
+      zHUrl = mprintf("<a %s href=\"%h\">", zExtra, zUrl);
+    }else{
+      zHUrl = mprintf("<a href=\"%h\">", zUrl);
+    }
     fossil_free(zUrl);
     return zHUrl;
   }
   needHrefJs = 1;
-  return mprintf("<a %s data-href='%z' href='%R/honeypot'>",
-                  zExtra, zUrl);
+  if( zExtra==0 ){
+    return mprintf("<a data-href='%z' href='%R/honeypot'>", zUrl);
+  }else{
+    return mprintf("<a %s data-href='%z' href='%R/honeypot'>",
+                   zExtra, zUrl);
+  }
 }
 char *chref(const char *zExtra, const char *zFormat, ...){
   char *zUrl;
@@ -154,7 +174,7 @@ char *chref(const char *zExtra, const char *zFormat, ...){
   zUrl = vmprintf(zFormat, ap);
   va_end(ap);
   if( g.perm.Hyperlink && !g.javascriptHyperlink ){
-    char *zHUrl = mprintf("<a %s href=\"%h\">", zExtra, zUrl);
+    char *zHUrl = mprintf("<a class=\"%s\" href=\"%h\">", zExtra, zUrl);
     fossil_free(zUrl);
     return zHUrl;
   }
@@ -305,6 +325,13 @@ void style_submenu_sql(
   }
 }
 
+/*
+** Disable or enable the submenu
+*/
+void style_submenu_enable(int onOff){
+  submenuEnable = onOff;
+}
+
 
 /*
 ** Compare two submenu items for sorting purposes
@@ -364,6 +391,149 @@ static void image_url_var(const char *zImageName){
   free(zConfigName);
 }
 
+/*
+** Output TEXT with a click-to-copy button next to it. Loads the copybtn.js
+** Javascript module, and generates HTML elements with the following IDs:
+**
+**    TARGETID:       The <span> wrapper around TEXT.
+**    copy-TARGETID:  The <span> for the copy button.
+**
+** If the FLIPPED argument is non-zero, the copy button is displayed after TEXT.
+**
+** The COPYLENGTH argument defines the length of the substring of TEXT copied to
+** clipboard:
+**
+**    <= 0:   No limit (default if the argument is omitted).
+**    >= 3:   Truncate TEXT after COPYLENGTH (single-byte) characters.
+**       1:   Use the "hash-digits" setting as the limit.
+**       2:   Use the length appropriate for URLs as the limit (defined at
+**            compile-time by FOSSIL_HASH_DIGITS_URL, defaults to 16).
+*/
+char *style_copy_button(
+  int bOutputCGI,         /* Don't return result, but send to cgi_printf(). */
+  const char *zTargetId,  /* The TARGETID argument. */
+  int bFlipped,           /* The FLIPPED argument. */
+  int cchLength,          /* The COPYLENGTH argument. */
+  const char *zTextFmt,   /* Formatting of the TEXT argument (htmlized). */
+  ...                     /* Formatting parameters of the TEXT argument. */
+){
+  va_list ap;
+  char *zText;
+  char *zResult = 0;
+  va_start(ap,zTextFmt);
+  zText = vmprintf(zTextFmt/*works-like:?*/,ap);
+  va_end(ap);
+  if( cchLength==1 ) cchLength = hash_digits(0);
+  else if( cchLength==2 ) cchLength = hash_digits(1);
+  if( !bFlipped ){
+    const char *zBtnFmt =
+      "<span class=\"nobr\">"
+      "<span "
+      "class=\"copy-button\" "
+      "id=\"copy-%h\" "
+      "data-copytarget=\"%h\" "
+      "data-copylength=\"%d\">"
+      "</span>"
+      "<span id=\"%h\">"
+      "%s"
+      "</span>"
+      "</span>";
+    if( bOutputCGI ){
+      cgi_printf(
+                  zBtnFmt/*works-like:"%h%h%d%h%s"*/,
+                  zTargetId,zTargetId,cchLength,zTargetId,zText);
+    }else{
+      zResult = mprintf(
+                  zBtnFmt/*works-like:"%h%h%d%h%s"*/,
+                  zTargetId,zTargetId,cchLength,zTargetId,zText);
+    }
+  }else{
+    const char *zBtnFmt =
+      "<span class=\"nobr\">"
+      "<span id=\"%h\">"
+      "%s"
+      "</span>"
+      "<span "
+      "class=\"copy-button copy-button-flipped\" "
+      "id=\"copy-%h\" "
+      "data-copytarget=\"%h\" "
+      "data-copylength=\"%d\">"
+      "</span>"
+      "</span>";
+    if( bOutputCGI ){
+      cgi_printf(
+                  zBtnFmt/*works-like:"%h%s%h%h%d"*/,
+                  zTargetId,zText,zTargetId,zTargetId,cchLength);
+    }else{
+      zResult = mprintf(
+                  zBtnFmt/*works-like:"%h%s%h%h%d"*/,
+                  zTargetId,zText,zTargetId,zTargetId,cchLength);
+    }
+  }
+  free(zText);
+  style_copybutton_control();
+  return zResult;
+}
+
+/*
+** Return a random nonce that is stored in static space.  For a particular
+** run, the same nonce is always returned.
+*/
+char *style_nonce(void){
+  static char zNonce[52];
+  if( zNonce[0]==0 ){
+    unsigned char zSeed[24];
+    sqlite3_randomness(24, zSeed);
+    encode16(zSeed,(unsigned char*)zNonce,24);
+  }
+  return zNonce;
+}
+
+/*
+** Return the default Content Security Policy (CSP) string.
+** If the toHeader argument is true, then also add the
+** CSP to the HTTP reply header.
+**
+** The CSP comes from the "default-csp" setting if it exists and
+** is non-empty.  If that setting is an empty string, then the following
+** default is used instead:
+**
+**     default-src 'self' data:;
+**     script-src 'self' 'nonce-$nonce';
+**     style-src 'self' 'unsafe-inline';
+**
+** The text '$nonce' is replaced by style_nonce() if and whereever it
+** occurs in the input string.
+**
+** The string returned is obtained from fossil_malloc() and
+** should be released by the caller.
+*/
+char *style_csp(int toHeader){
+  static const char zBackupCSP[] = 
+   "default-src 'self' data:; "
+   "script-src 'self' 'nonce-$nonce'; "
+   "style-src 'self' 'unsafe-inline'";
+  const char *zFormat = db_get("default-csp","");
+  Blob csp;
+  char *zNonce;
+  char *zCsp;
+  if( zFormat[0]==0 ){
+    zFormat = zBackupCSP;
+  }
+  blob_init(&csp, 0, 0);
+  while( zFormat[0] && (zNonce = strstr(zFormat,"$nonce"))!=0 ){
+    blob_append(&csp, zFormat, (int)(zNonce - zFormat));
+    blob_append(&csp, style_nonce(), -1);
+    zFormat = zNonce + 6;
+  }
+  blob_append(&csp, zFormat, -1);
+  zCsp = blob_str(&csp);
+  if( toHeader ){
+    cgi_printf_header("Content-Security-Policy: %s\r\n", zCsp);
+  }
+  return zCsp;
+}
+
 /*
 ** Default HTML page header text through <body>.  If the repository-specific
 ** header template lacks a <body> tag, then all of the following is
@@ -373,17 +543,54 @@ static char zDfltHeader[] =
 @ <html>
 @ <head>
 @ <base href="$baseurl/$current_page" />
-@ <meta http-equiv="Content-Security-Policy" \
-@  content="default-src 'self' data: 'unsafe-inline'" />
+@ <meta http-equiv="Content-Security-Policy" content="$default_csp" />
+@ <meta name="viewport" content="width=device-width, initial-scale=1.0">
 @ <title>$<project_name>: $<title></title>
 @ <link rel="alternate" type="application/rss+xml" title="RSS Feed" \
 @  href="$home/timeline.rss" />
-@ <link rel="stylesheet" href="$stylesheet_url" type="text/css" \
-@  media="screen" />
+@ <link rel="stylesheet" href="$stylesheet_url" type="text/css" />
 @ </head>
 @ <body>
 ;
 
+/*
+** Initialize all the default TH1 variables
+*/
+static void style_init_th1_vars(const char *zTitle){
+  const char *zNonce = style_nonce();
+  char *zDfltCsp;
+
+  zDfltCsp = style_csp(1);
+  /*
+  ** Do not overwrite the TH1 variable "default_csp" if it exists, as this
+  ** allows it to be properly overridden via the TH1 setup script (i.e. it
+  ** is evaluated before the header is rendered).
+  */
+  Th_MaybeStore("default_csp", zDfltCsp);
+  fossil_free(zDfltCsp);
+  Th_Store("nonce", zNonce);
+  Th_Store("project_name", db_get("project-name","Unnamed Fossil Project"));
+  Th_Store("project_description", db_get("project-description",""));
+  if( zTitle ) Th_Store("title", zTitle);
+  Th_Store("baseurl", g.zBaseURL);
+  Th_Store("secureurl", fossil_wants_https(1)? g.zHttpsURL: g.zBaseURL);
+  Th_Store("home", g.zTop);
+  Th_Store("index_page", db_get("index-page","/home"));
+  if( local_zCurrentPage==0 ) style_set_current_page("%T", g.zPath);
+  Th_Store("current_page", local_zCurrentPage);
+  Th_Store("csrf_token", g.zCsrfToken);
+  Th_Store("release_version", RELEASE_VERSION);
+  Th_Store("manifest_version", MANIFEST_VERSION);
+  Th_Store("manifest_date", MANIFEST_DATE);
+  Th_Store("compiler_name", COMPILER_NAME);
+  url_var("stylesheet", "css", "style.css");
+  image_url_var("logo");
+  image_url_var("background");
+  if( !login_is_nobody() ){
+    Th_Store("login", g.zLogin);
+  }
+}
+
 /*
 ** Draw the header.
 */
@@ -404,27 +611,8 @@ void style_header(const char *zTitleFormat, ...){
   if( g.thTrace ) Th_Trace("BEGIN_HEADER<br />\n", -1);
 
   /* Generate the header up through the main menu */
-  Th_Store("project_name", db_get("project-name","Unnamed Fossil Project"));
-  Th_Store("project_description", db_get("project-description",""));
-  Th_Store("title", zTitle);
-  Th_Store("baseurl", g.zBaseURL);
-  Th_Store("secureurl", login_wants_https_redirect()? g.zHttpsURL: g.zBaseURL);
-  Th_Store("home", g.zTop);
-  Th_Store("index_page", db_get("index-page","/home"));
-  if( local_zCurrentPage==0 ) style_set_current_page("%T", g.zPath);
-  Th_Store("current_page", local_zCurrentPage);
-  Th_Store("csrf_token", g.zCsrfToken);
-  Th_Store("release_version", RELEASE_VERSION);
-  Th_Store("manifest_version", MANIFEST_VERSION);
-  Th_Store("manifest_date", MANIFEST_DATE);
-  Th_Store("compiler_name", COMPILER_NAME);
-  url_var("stylesheet", "css", "style.css");
-  image_url_var("logo");
-  image_url_var("background");
-  if( !login_is_nobody() ){
-    Th_Store("login", g.zLogin);
-  }
-  if( sqlite3_strlike("%<body>%", zHeader, 0)!=0 ){
+  style_init_th1_vars(zTitle);
+  if( sqlite3_strlike("%<body%", zHeader, 0)!=0 ){
     Th_Render(zDfltHeader);
   }
   if( g.thTrace ) Th_Trace("BEGIN_HEADER_SCRIPT<br />\n", -1);
@@ -435,6 +623,11 @@ void style_header(const char *zTitleFormat, ...){
   g.cgiOutput = 1;
   headerHasBeenGenerated = 1;
   sideboxUsed = 0;
+  if( g.perm.Debug && P("showqp") ){
+    @ <div class="debug">
+    cgi_print_all(0, 0);
+    @ </div>
+  }
 }
 
 #if INTERFACE
@@ -492,39 +685,103 @@ void style_table_sorter(void){
 }
 
 /*
-** Indicate that the table-sorting javascript is needed.
+** Indicate that the accordion javascript is needed.
+*/
+void style_accordion(void){
+  needAccordionJs = 1;
+}
+
+/*
+** Indicate that the timeline graph javascript is needed.
 */
 void style_graph_generator(void){
   needGraphJs = 1;
 }
 
+/*
+** Indicate that the copy button javascript is needed.
+*/
+void style_copybutton_control(void){
+  needCopyBtnJs = 1;
+}
+
 /*
 ** Generate code to load a single javascript file
 */
 void style_load_one_js_file(const char *zFile){
-  @ <script src='%R/builtin/%s(zFile)?id=%S(MANIFEST_UUID)'></script>
+  @ <script src='%R/builtin/%s(zFile)?id=%S(fossil_exe_id())'></script>
+}
+
+/*
+** All extra JS files to load.
+*/
+static const char *azJsToLoad[4];
+static int nJsToLoad = 0;
+
+/*
+** Register a new JS file to load at the end of the document.
+*/
+void style_load_js(const char *zName){
+  int i;
+  for(i=0; i<nJsToLoad; i++){
+    if( fossil_strcmp(zName, azJsToLoad[i])==0 ) return;
+  }
+  if( nJsToLoad>=sizeof(azJsToLoad)/sizeof(azJsToLoad[0]) ){
+    fossil_panic("too many JS files");
+  }
+  azJsToLoad[nJsToLoad++] = zName;
 }
 
 /*
 ** Generate code to load all required javascript files.
 */
 static void style_load_all_js_files(void){
+  int i;
   if( needHrefJs ){
     int nDelay = db_get_int("auto-hyperlink-delay",0);
-    int bMouseover;
-    /* Load up the page data */
-    bMouseover = (!g.isHuman || db_get_boolean("auto-hyperlink-ishuman",0))
-                 && db_get_boolean("auto-hyperlink-mouseover",0);
+    int bMouseover = db_get_boolean("auto-hyperlink-mouseover",0);
     @ <script id='href-data' type='application/json'>\
     @ {"delay":%d(nDelay),"mouseover":%d(bMouseover)}</script>
-    style_load_one_js_file("href.js");
+  }
+  @ <script nonce="%h(style_nonce())">
+  @ function debugMsg(msg){
+  @ var n = document.getElementById("debugMsg");
+  @ if(n){n.textContent=msg;}
+  @ }
+  if( needHrefJs ){
+    cgi_append_content(builtin_text("href.js"),-1);
   }
   if( needSortJs ){
-    style_load_one_js_file("sorttable.js");
+    cgi_append_content(builtin_text("sorttable.js"),-1);
   }
   if( needGraphJs ){
-    style_load_one_js_file("graph.js");
+    cgi_append_content(builtin_text("graph.js"),-1);
   }
+  if( needCopyBtnJs ){
+    cgi_append_content(builtin_text("copybtn.js"),-1);
+  }
+  if( needAccordionJs ){
+    cgi_append_content(builtin_text("accordion.js"),-1);
+  }
+  for(i=0; i<nJsToLoad; i++){
+    cgi_append_content(builtin_text(azJsToLoad[i]),-1);
+  }
+  if( blob_size(&blobOnLoad)>0 ){
+    @ window.onload = function(){
+    cgi_append_content(blob_buffer(&blobOnLoad), blob_size(&blobOnLoad));
+    cgi_append_content("\n}\n", -1);
+  }
+  @ </script>
+}
+
+/*
+** Extra JS to run after all content is loaded.
+*/
+void style_js_onload(const char *zFormat, ...){
+  va_list ap;
+  va_start(ap, zFormat);
+  blob_vappendf(&blobOnLoad, zFormat, ap);
+  va_end(ap);
 }
 
 /*
@@ -542,11 +799,12 @@ void style_footer(void){
   ** to the submenu while generating page text.
   */
   cgi_destination(CGI_HEADER);
-  if( nSubmenu+nSubmenuCtrl>0 ){
+  if( submenuEnable && nSubmenu+nSubmenuCtrl>0 ){
     int i;
     if( nSubmenuCtrl ){
       @ <form id='f01' method='GET' action='%R/%s(g.zPath)'>
       @ <input type='hidden' name='udc' value='1'>
+      cgi_tag_query_parameter("udc");
     }
     @ <div class="submenu">
     if( nSubmenu>0 ){
@@ -661,7 +919,7 @@ void style_footer(void){
       cgi_append_content(zAd, -1);
       @ </div>
     }
-    @ <div class="content">
+    @ <div class="content"><span id="debugMsg"></span>
   }
   cgi_destination(CGI_BODY);
 
@@ -781,6 +1039,25 @@ void contains_selector_cmd(void){
   blob_reset(&css);
 }
 
+/*
+** WEBPAGE: script.js
+**
+** Return the "Javascript" content for the current skin (if there is any)
+*/
+void page_script_js(void){
+  const char *zScript = skin_get("js");
+  if( P("test") ){
+    /* Render the script as plain-text for testing purposes, if the "test"
+    ** query parameter is present */
+    cgi_set_content_type("text/plain");
+  }else{
+    /* Default behavior is to return javascript */
+    cgi_set_content_type("application/javascript");
+  }
+  style_init_th1_vars(0);
+  Th_Render(zScript?zScript:"");
+}
+
 
 /*
 ** WEBPAGE: style.css
@@ -820,7 +1097,7 @@ void page_style_css(void){
   ** variables such as $baseurl.
   */
   Th_Store("baseurl", g.zBaseURL);
-  Th_Store("secureurl", login_wants_https_redirect()? g.zHttpsURL: g.zBaseURL);
+  Th_Store("secureurl", fossil_wants_https(1)? g.zHttpsURL: g.zBaseURL);
   Th_Store("home", g.zTop);
   image_url_var("logo");
   image_url_var("background");
@@ -866,6 +1143,39 @@ void page_builtin_text(void){
   cgi_set_content(&out);
 }
 
+/*
+** All possible capabilities
+*/
+static const char allCap[] = 
+  "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKL";
+
+/*
+** Compute the current login capabilities
+*/
+static char *find_capabilities(char *zCap){
+  int i, j;
+  char c;
+  for(i=j=0; (c = allCap[j])!=0; j++){
+    if( login_has_capability(&c, 1, 0) ) zCap[i++] = c;
+  }
+  zCap[i] = 0;
+  return zCap;
+}
+
+/*
+** Compute the current login capabilities that were
+** contributed by Anonymous
+*/
+static char *find_anon_capabilities(char *zCap){
+  int i, j;
+  char c;
+  for(i=j=0; (c = allCap[j])!=0; j++){
+    if( login_has_capability(&c, 1, LOGIN_ANON)
+      && !login_has_capability(&c, 1, 0) ) zCap[i++] = c;
+  }
+  zCap[i] = 0;
+  return zCap;
+}
 
 /*
 ** WEBPAGE: test_env
@@ -874,86 +1184,7 @@ void page_builtin_text(void){
 ** environment, for debugging and trouble-shooting purposes.
 */
 void page_test_env(void){
-  char c;
-  int i;
-  int showAll;
-  char zCap[30];
-  static const char *const azCgiVars[] = {
-    "COMSPEC", "DOCUMENT_ROOT", "GATEWAY_INTERFACE",
-    "HTTP_ACCEPT", "HTTP_ACCEPT_CHARSET", "HTTP_ACCEPT_ENCODING",
-    "HTTP_ACCEPT_LANGUAGE", "HTTP_AUTHENICATION",
-    "HTTP_CONNECTION", "HTTP_HOST",
-    "HTTP_IF_NONE_MATCH", "HTTP_IF_MODIFIED_SINCE",
-    "HTTP_USER_AGENT", "HTTP_REFERER", "PATH_INFO", "PATH_TRANSLATED",
-    "QUERY_STRING", "REMOTE_ADDR", "REMOTE_PORT",
-    "REMOTE_USER", "REQUEST_METHOD",
-    "REQUEST_URI", "SCRIPT_FILENAME", "SCRIPT_NAME", "SERVER_PROTOCOL",
-    "HOME", "FOSSIL_HOME", "USERNAME", "USER", "FOSSIL_USER",
-    "SQLITE_TMPDIR", "TMPDIR",
-    "TEMP", "TMP", "FOSSIL_VFS",
-    "FOSSIL_FORCE_TICKET_MODERATION", "FOSSIL_FORCE_WIKI_MODERATION",
-    "FOSSIL_TCL_PATH", "TH1_DELETE_INTERP", "TH1_ENABLE_DOCS",
-    "TH1_ENABLE_HOOKS", "TH1_ENABLE_TCL", "REMOTE_HOST"
-  };
-
-  login_check_credentials();
-  if( !g.perm.Admin && !g.perm.Setup && !db_get_boolean("test_env_enable",0) ){
-    login_needed(0);
-    return;
-  }
-  for(i=0; i<count(azCgiVars); i++) (void)P(azCgiVars[i]);
-  style_header("Environment Test");
-  showAll = PB("showall");
-  style_submenu_checkbox("showall", "Cookies", 0, 0);
-  style_submenu_element("Stats", "%R/stat");
-
-#if !defined(_WIN32)
-  @ uid=%d(getuid()), gid=%d(getgid())<br />
-#endif
-  @ g.zBaseURL = %h(g.zBaseURL)<br />
-  @ g.zHttpsURL = %h(g.zHttpsURL)<br />
-  @ g.zTop = %h(g.zTop)<br />
-  @ g.zPath = %h(g.zPath)<br />
-  for(i=0, c='a'; c<='z'; c++){
-    if( login_has_capability(&c, 1, 0) ) zCap[i++] = c;
-  }
-  zCap[i] = 0;
-  @ g.userUid = %d(g.userUid)<br />
-  @ g.zLogin = %h(g.zLogin)<br />
-  @ g.isHuman = %d(g.isHuman)<br />
-  if( g.nRequest ){
-    @ g.nRequest = %d(g.nRequest)<br />
-  }
-  if( g.nPendingRequest>1 ){
-    @ g.nPendingRequest = %d(g.nPendingRequest)<br />
-  }
-  @ capabilities = %s(zCap)<br />
-  for(i=0, c='a'; c<='z'; c++){
-    if( login_has_capability(&c, 1, LOGIN_ANON)
-         && !login_has_capability(&c, 1, 0) ) zCap[i++] = c;
-  }
-  zCap[i] = 0;
-  if( i>0 ){
-    @ anonymous-adds = %s(zCap)<br />
-  }
-  @ g.zRepositoryName = %h(g.zRepositoryName)<br />
-  @ load_average() = %f(load_average())<br />
-  @ cgi_csrf_safe(0) = %d(cgi_csrf_safe(0))<br />
-  @ <hr />
-  P("HTTP_USER_AGENT");
-  cgi_print_all(showAll);
-  if( showAll && blob_size(&g.httpHeader)>0 ){
-    @ <hr />
-    @ <pre>
-    @ %h(blob_str(&g.httpHeader))
-    @ </pre>
-  }
-  if( g.perm.Setup ){
-    const char *zRedir = P("redirect");
-    if( zRedir ) cgi_redirect(zRedir);
-  }
-  style_footer();
-  if( g.perm.Admin && P("err") ) fossil_fatal("%s", P("err"));
+  webpage_error("");
 }
 
 /*
@@ -964,3 +1195,106 @@ void honeypot_page(void){
   cgi_set_status(403, "Forbidden");
   @ <p>Please enable javascript or log in to see this content</p>
 }
+
+/*
+** Webpages that encounter an error due to missing or incorrect
+** query parameters can jump to this routine to render an error
+** message screen.
+**
+** For administators, or if the test_env_enable setting is true, then
+** details of the request environment are displayed.  Otherwise, just
+** the error message is shown.
+**
+** If zFormat is an empty string, then this is the /test_env page.
+*/
+void webpage_error(const char *zFormat, ...){
+  int showAll;
+  char *zErr = 0;
+  int isAuth = 0;
+  char zCap[100];
+
+  login_check_credentials();
+  if( g.perm.Admin || g.perm.Setup  || db_get_boolean("test_env_enable",0) ){
+    isAuth = 1;
+  }
+  cgi_load_environment();
+  if( zFormat[0] ){
+    va_list ap;
+    va_start(ap, zFormat);
+    zErr = vmprintf(zFormat, ap);
+    va_end(ap);
+    style_header("Bad Request");
+    @ <h1>/%h(g.zPath): %h(zErr)</h1>
+    showAll = 0;
+    cgi_set_status(500, "Bad Request");
+  }else if( !isAuth ){
+    login_needed(0);
+    return;
+  }else{
+    style_header("Environment Test");
+    showAll = PB("showall");
+    style_submenu_checkbox("showall", "Cookies", 0, 0);
+    style_submenu_element("Stats", "%R/stat");
+  }
+
+  if( isAuth ){
+  #if !defined(_WIN32)
+    @ uid=%d(getuid()), gid=%d(getgid())<br />
+  #endif
+    @ g.zBaseURL = %h(g.zBaseURL)<br />
+    @ g.zHttpsURL = %h(g.zHttpsURL)<br />
+    @ g.zTop = %h(g.zTop)<br />
+    @ g.zPath = %h(g.zPath)<br />
+    @ g.userUid = %d(g.userUid)<br />
+    @ g.zLogin = %h(g.zLogin)<br />
+    @ g.isHuman = %d(g.isHuman)<br />
+    if( g.nRequest ){
+      @ g.nRequest = %d(g.nRequest)<br />
+    }
+    if( g.nPendingRequest>1 ){
+      @ g.nPendingRequest = %d(g.nPendingRequest)<br />
+    }
+    @ capabilities = %s(find_capabilities(zCap))<br />
+    if( zCap[0] ){
+      @ anonymous-adds = %s(find_anon_capabilities(zCap))<br />
+    }
+    @ g.zRepositoryName = %h(g.zRepositoryName)<br />
+    @ load_average() = %f(load_average())<br />
+    @ cgi_csrf_safe(0) = %d(cgi_csrf_safe(0))<br />
+    @ fossil_exe_id() = %h(fossil_exe_id())<br />
+    @ <hr />
+    P("HTTP_USER_AGENT");
+    cgi_print_all(showAll, 0);
+    if( showAll && blob_size(&g.httpHeader)>0 ){
+      @ <hr />
+      @ <pre>
+      @ %h(blob_str(&g.httpHeader))
+      @ </pre>
+    }
+  }
+  style_footer();
+  if( zErr ){
+    cgi_reply();
+    fossil_exit(1);
+  }
+}
+
+/*
+** Generate a Not Yet Implemented error page.
+*/
+void webpage_not_yet_implemented(void){
+  webpage_error("Not yet implemented");
+}
+
+/*
+** Generate a webpage for a webpage_assert().
+*/
+void webpage_assert_page(const char *zFile, int iLine, const char *zExpr){
+  fossil_warning("assertion fault at %s:%d - %s", zFile, iLine, zExpr);
+  cgi_reset_content();
+  webpage_error("assertion fault at %s:%d - %s", zFile, iLine, zExpr);
+}
+
+#if INTERFACE
+# define webpage_assert(T) if(!(T)){webpage_assert_page(__FILE__,__LINE__,#T);}
+#endif
