@@ -254,7 +254,7 @@ static void add_pax_header(
   blob_appendf(&tball.pax, "%d %s=%*.*s\n", blen, zField, nValue, nValue, zValue);
   /* this _must_ be right */
   if(blob_size(&tball.pax) != blen){
-    fossil_fatal("internal error: PAX tar header has bad length");
+    fossil_panic("internal error: PAX tar header has bad length");
   }
 }
 
@@ -445,7 +445,7 @@ void test_tarball_cmd(void){
     Blob file;
     blob_zero(&file);
     blob_read_from_file(&file, g.argv[i], eFType);
-    tar_add_file(g.argv[i], &file, file_perm(0,0), file_mtime(0,0));
+    tar_add_file(g.argv[i], &file, file_perm(0,eFType), file_mtime(0,eFType));
     blob_reset(&file);
   }
   tar_finish(&zip);
@@ -810,6 +810,7 @@ void tarball_page(void){
   glob_free(pExclude);
   fossil_free(zName);
   fossil_free(zRid);
+  g.zOpenRevision = 0;
   blob_reset(&cacheKey);
   cgi_set_content(&tarball);
   cgi_set_content_type("application/x-compressed");
