@@ -40,6 +40,7 @@
 #define URL_ASK_REMEMBER_PW  0x004  /* Ask whether to remember prompted pw */
 #define URL_REMEMBER_PW      0x008  /* Should remember pw */
 #define URL_PROMPTED         0x010  /* Prompted for PW already */
+#define URL_OMIT_USER        0x020  /* Omit the user name from URL */
 
 /*
 ** The URL related data used with this subsystem.
@@ -67,16 +68,6 @@ struct UrlData {
 #endif /* INTERFACE */
 
 
-/*
-** Convert a string to lower-case.
-*/
-static void url_tolower(char *z){
-  while( *z ){
-     *z = fossil_tolower(*z);
-     z++;
-  }
-}
-
 /*
 ** Parse the given URL.  Populate members of the provided UrlData structure
 ** as follows:
@@ -155,7 +146,11 @@ void url_parse_local(
       if( pUrlData->isSsh ){
         urlFlags &= ~URL_ASK_REMEMBER_PW;
       }
-      zLogin = mprintf("%t@", pUrlData->user);
+      if( urlFlags & URL_OMIT_USER ){
+        zLogin = mprintf("");
+      }else{
+        zLogin = mprintf("%t@", pUrlData->user);
+      }
       for(j=i+1; (c=zUrl[j])!=0 && c!='/' && c!=':'; j++){}
       pUrlData->name = mprintf("%.*s", j-i-1, &zUrl[i+1]);
       i = j;
@@ -174,7 +169,7 @@ void url_parse_local(
       }
       zLogin = mprintf("");
     }
-    url_tolower(pUrlData->name);
+    fossil_strtolwr(pUrlData->name);
     if( c==':' ){
       pUrlData->port = 0;
       i++;
@@ -241,12 +236,12 @@ void url_parse_local(
       i = 5;
     }
     zFile = mprintf("%s", &zUrl[i]);
-  }else if( file_isfile(zUrl) ){
+  }else if( file_isfile(zUrl, ExtFILE) ){
     pUrlData->isFile = 1;
     zFile = mprintf("%s", zUrl);
-  }else if( file_isdir(zUrl)==1 ){
+  }else if( file_isdir(zUrl, ExtFILE)==1 ){
     zFile = mprintf("%s/FOSSIL", zUrl);
-    if( file_isfile(zFile) ){
+    if( file_isfile(zFile, ExtFILE) ){
       pUrlData->isFile = 1;
     }else{
       free(zFile);
@@ -271,7 +266,7 @@ void url_parse_local(
   }else if( pUrlData->user!=0 && pUrlData->passwd==0 && (urlFlags & URL_PROMPT_PW) ){
     url_prompt_for_password_local(pUrlData);
   }else if( pUrlData->user!=0 && ( urlFlags & URL_ASK_REMEMBER_PW ) ){
-    if( isatty(fileno(stdin)) ){
+    if( isatty(fileno(stdin)) && ( urlFlags & URL_REMEMBER_PW )==0 ){
       if( save_password_prompt(pUrlData->passwd) ){
         pUrlData->flags = urlFlags |= URL_REMEMBER_PW;
       }else{
@@ -367,14 +362,28 @@ static const char *zProxyOpt = 0;
 **
 **    --proxy URL|off
 **
-** This also happens to be a convenient function to use to look for
-** the --nosync option that will temporarily disable the "autosync"
-** feature.
+** The original purpose of this routine is the above.  But this
+** also happens to be a convenient place to look for other
+** network-related options:
+**
+**    --nosync             Temporarily disable "autosync"
+**
+**    --ipv4               Disallow IPv6.  Use only IPv4.
+**
+**    --accept-any-cert    Disable server SSL cert validation. Accept
+**                         any SSL cert that the server provides.
+**                         WARNING: this option opens you up to
+**                         forged-DNS and man-in-the-middle attacks!
 */
 void url_proxy_options(void){
   zProxyOpt = find_option("proxy", 0, 1);
   if( find_option("nosync",0,0) ) g.fNoSync = 1;
   if( find_option("ipv4",0,0) ) g.fIPv4 = 1;
+#ifdef FOSSIL_ENABLE_SSL
+  if( find_option("accept-any-cert",0,0) ){
+    ssl_disable_cert_verification();
+  }
+#endif /* FOSSIL_ENABLE_SSL */
 }
 
 /*
@@ -456,8 +465,8 @@ void url_initialize(HQuery *p, const char *zBase){
 */
 void url_reset(HQuery *p){
   blob_reset(&p->url);
-  fossil_free(p->azName);
-  fossil_free(p->azValue);
+  fossil_free((void *)p->azName);
+  fossil_free((void *)p->azValue);
   url_initialize(p, p->zBase);
 }
 
@@ -482,8 +491,10 @@ void url_add_parameter(HQuery *p, const char *zName, const char *zValue){
   if( zValue==0 ) return;
   if( i>=p->nAlloc ){
     p->nAlloc = p->nAlloc*2 + 10;
-    p->azName = fossil_realloc(p->azName, sizeof(p->azName[0])*p->nAlloc);
-    p->azValue = fossil_realloc(p->azValue, sizeof(p->azValue[0])*p->nAlloc);
+    p->azName = fossil_realloc((void *)p->azName,
+                               sizeof(p->azName[0])*p->nAlloc);
+    p->azValue = fossil_realloc((void *)p->azValue,
+                                sizeof(p->azValue[0])*p->nAlloc);
   }
   p->azName[i] = zName;
   p->azValue[i] = zValue;
@@ -527,6 +538,7 @@ char *url_render(
   if( zName1 && zValue1 ){
     blob_appendf(&p->url, "%s%s", zSep, zName1);
     if( zValue1[0] ) blob_appendf(&p->url, "=%T", zValue1);
+    zSep = "&";
   }
   if( zName2 && zValue2 ){
     blob_appendf(&p->url, "%s%s", zSep, zName2);
