@@ -23,16 +23,28 @@
 # is the name of the executable to be tested.
 #
 
+# We use some things introduced in 8.6 such as lmap.  auto.def should
+# have found us a suitable Tcl installation.
+package require Tcl 8.6
+
 set testfiledir [file normalize [file dirname [info script]]]
 set testrundir [pwd]
 set testdir [file normalize [file dirname $argv0]]
 set fossilexe [file normalize [lindex $argv 0]]
+set is_windows [expr {$::tcl_platform(platform) eq "windows"}]
 
-if {$tcl_platform(platform) eq "windows" && \
-    [string length [file extension $fossilexe]] == 0} {
-  append fossilexe .exe
+if {$::is_windows} {
+  if {[string length [file extension $fossilexe]] == 0} {
+    append fossilexe .exe
+  }
+  set outside_fossil_repo [expr ![file exists "$::testfiledir\\..\\_FOSSIL_"]]
+} else {
+  set outside_fossil_repo [expr ![file exists "$::testfiledir/../.fslckout"]]
 }
 
+catch {exec $::fossilexe changes --changed} res
+set dirty_ckout [string length $res]
+
 set argv [lrange $argv 1 end]
 
 set i [lsearch $argv -keep]
@@ -115,6 +127,17 @@ proc protOut {msg {noQuiet 0}} {
   }
 }
 
+# write a dict with just enough formatting
+# to make it human readable
+#
+proc protOutDict {dict {pattern *}} {
+   set longest [tcl::mathfunc::max 0 {*}[lmap key [dict keys $dict $pattern] {string length $key}]]
+   dict for {key value} $dict {
+      protOut [format "%-${longest}s = %s" $key $value]
+   }
+}
+
+
 # Run the Fossil program with the specified arguments.
 #
 # Consults the VERBOSE global variable to determine if
@@ -138,9 +161,16 @@ proc fossil_maybe_answer {answer args} {
   global fossilexe
   set cmd $fossilexe
   set expectError 0
-  if {[lindex $args end] eq "-expectError"} {
+  set index [lsearch -exact $args -expectError]
+  if {$index != -1} {
     set expectError 1
-    set args [lrange $args 0 end-1]
+    set args [lreplace $args $index $index]
+  }
+  set keepNewline 0
+  set index [lsearch -exact $args -keepNewline]
+  if {$index != -1} {
+    set keepNewline 1
+    set args [lreplace $args $index $index]
   }
   foreach a $args {
     lappend cmd $a
@@ -152,10 +182,18 @@ proc fossil_maybe_answer {answer args} {
     protOut $answer
     set prompt_file [file join $::tempPath fossil_prompt_answer]
     write_file $prompt_file $answer\n
-    set rc [catch {eval exec $cmd <$prompt_file} result]
+    if {$keepNewline} {
+      set rc [catch {eval exec -keepnewline $cmd <$prompt_file} result]
+    } else {
+      set rc [catch {eval exec $cmd <$prompt_file} result]
+    }
     file delete $prompt_file
   } else {
-    set rc [catch {eval exec $cmd} result]
+    if {$keepNewline} {
+      set rc [catch {eval exec -keepnewline $cmd} result]
+    } else {
+      set rc [catch {eval exec $cmd} result]
+    }
   }
   global RESULT CODE
   set CODE $rc
@@ -189,6 +227,133 @@ proc write_file_indented {filename txt} {
   write_file $filename [string trim [string map [list "\n  " \n] $txt]]\n
 }
 
+# Returns the list of all supported versionable settings.
+#
+proc get_versionable_settings {} {
+  #
+  # TODO: If the list of supported versionable settings in "db.c" is modified,
+  #       this list (and procedure) most likely needs to be modified as well.
+  #
+  set result [list \
+      allow-symlinks \
+      binary-glob \
+      clean-glob \
+      crlf-glob \
+      crnl-glob \
+      dotfiles \
+      empty-dirs \
+      encoding-glob \
+      ignore-glob \
+      keep-glob \
+      manifest \
+      th1-setup \
+      th1-uri-regexp]
+
+  fossil test-th-eval "hasfeature tcl"
+
+  if {[normalize_result] eq "1"} {
+    lappend result tcl-setup
+  }
+
+  return [lsort -dictionary $result]
+}
+
+# Returns the list of all supported settings.
+#
+proc get_all_settings {} {
+  #
+  # TODO: If the list of supported settings in "db.c" is modified, this list
+  #       (and procedure) most likely needs to be modified as well.
+  #
+  set result [list \
+      access-log \
+      admin-log \
+      allow-symlinks \
+      auto-captcha \
+      auto-hyperlink \
+      auto-shun \
+      autosync \
+      autosync-tries \
+      backoffice-disable \
+      backoffice-logfile \
+      backoffice-nodelay \
+      binary-glob \
+      case-sensitive \
+      clean-glob \
+      clearsign \
+      comment-format \
+      crlf-glob \
+      crnl-glob \
+      default-perms \
+      diff-binary \
+      diff-command \
+      dont-push \
+      dotfiles \
+      editor \
+      email-self \
+      email-send-command \
+      email-send-db \
+      email-send-dir \
+      email-send-method \
+      email-send-relayhost \
+      empty-dirs \
+      encoding-glob \
+      exec-rel-paths \
+      gdiff-command \
+      gmerge-command \
+      hash-digits \
+      http-port \
+      https-login \
+      ignore-glob \
+      keep-glob \
+      localauth \
+      main-branch \
+      manifest \
+      max-loadavg \
+      max-upload \
+      mtime-changes \
+      pgp-command \
+      proxy \
+      relative-paths \
+      repo-cksum \
+      repolist-skin \
+      self-register \
+      ssh-command \
+      ssl-ca-location \
+      ssl-identity \
+      tclsh \
+      th1-setup \
+      th1-uri-regexp \
+      uv-sync \
+      web-browser]
+
+  fossil test-th-eval "hasfeature legacyMvRm"
+
+  if {[normalize_result] eq "1"} {
+    lappend result mv-rm-files
+  }
+
+  fossil test-th-eval "hasfeature tcl"
+
+  if {[normalize_result] eq "1"} {
+    lappend result tcl tcl-setup
+  }
+
+  fossil test-th-eval "hasfeature th1Docs"
+
+  if {[normalize_result] eq "1"} {
+    lappend result th1-docs
+  }
+
+  fossil test-th-eval "hasfeature th1Hooks"
+
+  if {[normalize_result] eq "1"} {
+    lappend result th1-hooks
+  }
+
+  return [lsort -dictionary $result]
+}
+
 # Return true if two files are the same
 #
 proc same_file {a b} {
@@ -199,13 +364,36 @@ proc same_file {a b} {
   return [expr {$x==$y}]
 }
 
+# Return true if two strings refer to the
+# same uuid. That is, the shorter is a prefix
+# of the longer.
+#
+proc same_uuid {a b} {
+  set na [string length $a]
+  set nb [string length $b]
+  if {$na == $nb} {
+    return [expr {$a eq $b}]
+  }
+  if {$na < $nb} then {
+    return [string match "$a*" $b]
+  }
+  return [string match "$b*" $a]
+}
+
+# Return a prefix of a uuid, defaulting to 10 chars.
+#
+proc short_uuid {uuid {len 10}} {
+  string range $uuid 0 $len-1
+}
+
+
 proc require_no_open_checkout {} {
   if {[info exists ::env(FOSSIL_TEST_DANGEROUS_IGNORE_OPEN_CHECKOUT)] && \
       $::env(FOSSIL_TEST_DANGEROUS_IGNORE_OPEN_CHECKOUT) eq "YES_DO_IT"} {
     return
   }
   catch {exec $::fossilexe info} res
-  if {![regexp {use --repository} $res]} {
+  if {[regexp {local-root:} $res]} {
     set projectName <unknown>
     set localRoot <unknown>
     regexp -line -- {^project-name: (.*)$} $res dummy projectName
@@ -277,7 +465,7 @@ proc test_cleanup {} {
 
 proc delete_temporary_home {} {
   if {$::KEEP} {return}; # All cleanup disabled?
-  if {$::tcl_platform(platform) eq "windows"} {
+  if {$::is_windows} {
     robust_delete [file join $::tempHomePath _fossil]
   } else {
     robust_delete [file join $::tempHomePath .fossil]
@@ -323,17 +511,18 @@ please set TEMP variable in environment, error: $error"
     exec $::fossilexe open $filename
     exec $::fossilexe set mtime-changes off
   }
+  return $repoPath
 }
 
 # This procedure only returns non-zero if the Tcl integration feature was
 # enabled at compile-time and is now enabled at runtime.
 proc is_tcl_usable_by_fossil {} {
   fossil test-th-eval "hasfeature tcl"
-  if {$::RESULT ne "1"} {return 0}
+  if {[normalize_result] ne "1"} {return 0}
   fossil test-th-eval "setting tcl"
-  if {$::RESULT eq "1"} {return 1}
+  if {[normalize_result] eq "1"} {return 1}
   fossil test-th-eval --open-config "setting tcl"
-  if {$::RESULT eq "1"} {return 1}
+  if {[normalize_result] eq "1"} {return 1}
   return [info exists ::env(TH1_ENABLE_TCL)]
 }
 
@@ -341,21 +530,32 @@ proc is_tcl_usable_by_fossil {} {
 # at compile-time and is now enabled at runtime.
 proc are_th1_hooks_usable_by_fossil {} {
   fossil test-th-eval "hasfeature th1Hooks"
-  if {$::RESULT ne "1"} {return 0}
+  if {[normalize_result] ne "1"} {return 0}
   fossil test-th-eval "setting th1-hooks"
-  if {$::RESULT eq "1"} {return 1}
+  if {[normalize_result] eq "1"} {return 1}
   fossil test-th-eval --open-config "setting th1-hooks"
-  if {$::RESULT eq "1"} {return 1}
+  if {[normalize_result] eq "1"} {return 1}
   return [info exists ::env(TH1_ENABLE_HOOKS)]
 }
 
-# This (rarely used) procedure is designed to run a test within the Fossil
-# source checkout (e.g. one that does NOT modify any state), while saving
-# and restoring the current directory (e.g. one used when running a test
-# file outside of the Fossil source checkout).  Please do NOT use this
-# procedure unless you are absolutely sure it does not modify the state of
-# the repository or source checkout in any way.
+# Run the given command script inside the Fossil source repo checkout.
 #
+# Callers of this function must ensure two things:
+#
+# 1. This test run is in fact being done from within a Fossil repo
+#    checkout directory.  If you are unsure, test $::outside_fossil_repo
+#    or call one of the test_* wrappers below which do that for you.
+#
+#    As a rule, you should not be calling this function directly!
+#
+# 2. This test run is being done from a repo checkout directory that
+#    doesn't have any uncommitted changes.  If it does, that affects the
+#    output of any test based on the output of "fossil status",
+#    "... diff", etc., which is likely to make the test appear to fail.
+#    If you must call this function directly, test $::dirty_ckout and
+#    skip the call if it's true.  The test_* wrappers do this for you.
+#
+# 3. The test does NOT modify the Fossil checkout tree in any way.
 proc run_in_checkout { script {dir ""} } {
   if {[string length $dir] == 0} {set dir $::testfiledir}
   set savedPwd [pwd]; cd $dir
@@ -366,6 +566,35 @@ proc run_in_checkout { script {dir ""} } {
   return -code $code $result
 }
 
+# Wrapper for the above function pair.  The tscript parameter is an
+# optional post-run test script.  Some callers choose instead to put
+# the tests inline with the rscript commands.
+#
+# Be sure to adhere to the requirements of run_in_checkout!
+proc test_block_in_checkout { name rscript {tscript ""} } {
+  if {$::outside_fossil_repo || $::dirty_ckout} {
+    set $::CODE 0
+    set $::RESULT ""
+  } else {
+    run_in_checkout $rscript
+    if {[string length $tscript] == 0} {
+      return ""
+    } else {
+      set code [catch {
+        uplevel 1 $tscript
+      } result]
+      return -code $code $result
+    }
+  }
+}
+
+# Single-test wrapper for the above.
+proc test_in_checkout { name rscript tscript } {
+  return test_block_in_checkout name rscript {
+    test $name $tscript
+  }
+}
+
 # Normalize file status lists (like those returned by 'fossil changes')
 # so they can be compared using simple string comparison
 #
@@ -449,10 +678,10 @@ proc getTemporaryPath {} {
   foreach name $names {
     set value [getEnvironmentVariable $name]
 
-    if {[string length $value] > 0} then {
+    if {[string length $value] > 0} {
       set value [file normalize $value]
 
-      if {[file exists $value] && [file isdirectory $value]} then {
+      if {[file exists $value] && [file isdirectory $value]} {
         return $value
       }
     }
@@ -461,10 +690,10 @@ proc getTemporaryPath {} {
   #
   # NOTE: On non-Windows systems, fallback to /tmp if it is usable.
   #
-  if {$::tcl_platform(platform) ne "windows"} {
+  if {!$::is_windows} {
     set value /tmp
 
-    if {[file exists $value] && [file isdirectory $value]} then {
+    if {[file exists $value] && [file isdirectory $value]} {
       return $value
     }
   }
@@ -557,7 +786,7 @@ proc test {name expr {constraints ""}} {
       protOut "test $name FAILED!" 1
       if {$::QUIET} {protOut "RESULT: $RESULT" 1}
       lappend bad_test $name
-      if {$::HALT} exit
+      if {$::HALT} {exit 1}
     }
   }
 }
@@ -622,6 +851,98 @@ proc random_changes {body blocksize count index prob} {
   return [string range $out 1 end]
 }
 
+# This procedure executes the "fossil server" command.  The return value
+# is a list comprised of the new process identifier and the port on which
+# the server started.  The varName argument refers to a variable
+# where the "stop argument" is to be stored.  This value must eventually be
+# passed to the [test_stop_server] procedure.
+proc test_start_server { repository {varName ""} } {
+  global fossilexe tempPath
+  set command [list exec $fossilexe server --localhost]
+  if {[string length $varName] > 0} {
+    upvar 1 $varName stopArg
+  }
+  if {$::is_windows} {
+    set stopArg [file join [getTemporaryPath] [appendArgs \
+        [string trim [clock seconds] -] _ [getSeqNo] .stopper]]
+    lappend command --stopper $stopArg
+  }
+  set outFileName [file join $tempPath [appendArgs \
+      fossil_server_ [string trim [clock seconds] -] _ \
+      [getSeqNo]]].out
+  lappend command $repository >&$outFileName &
+  set pid [eval $command]
+  if {!$::is_windows} {
+    set stopArg $pid
+  }
+  after 1000; # output might not be there yet
+  set output [read_file $outFileName]
+  if {![regexp {Listening.*TCP port (\d+)} $output dummy port]} {
+    puts stdout "Could not detect Fossil server port, using default..."
+    set port 8080; # return the default port just in case
+  }
+  return [list $pid $port $outFileName]
+}
+
+# This procedure stops a Fossil server instance that was previously started
+# by the [test_start_server] procedure.  The value of the "stop argument"
+# will vary by platform as will the exact method used to stop the server.
+# The fileName argument is the name of a temporary output file to delete.
+proc test_stop_server { stopArg pid fileName } {
+  if {$::is_windows} {
+    #
+    # NOTE: On Windows, the "stop argument" must be the name of a file
+    #       that does NOT already exist.
+    #
+    if {[string length $stopArg] > 0 && \
+        ![file exists $stopArg] && \
+        [catch {write_file $stopArg [clock seconds]}] == 0} {
+      while {1} {
+        if {[catch {
+          #
+          # NOTE: Using the TaskList utility requires Windows XP or
+          #       later.
+          #
+          exec tasklist.exe /FI "PID eq $pid"
+        } result] != 0 || ![regexp -- " $pid " $result]} {
+          break
+        }
+        after 1000; # wait a bit...
+      }
+      file delete $stopArg
+      if {[string length $fileName] > 0} {
+        file delete $fileName
+      }
+      return true
+    }
+  } else {
+    #
+    # NOTE: On Unix, the "stop argument" must be an integer identifier
+    #       that refers to an existing process.
+    #
+    if {[regexp {^(?:-)?\d+$} $stopArg] && \
+        [catch {exec kill -TERM $stopArg}] == 0} {
+      while {1} {
+        if {[catch {
+          #
+          # TODO: Is this portable to all the supported variants of
+          #       Unix?  It should be, it's POSIX.
+          #
+          exec ps -p $pid
+        } result] != 0 || ![regexp -- "(?:^$pid| $pid) " $result]} {
+          break
+        }
+        after 1000; # wait a bit...
+      }
+      if {[string length $fileName] > 0} {
+        file delete $fileName
+      }
+      return true
+    }
+  }
+  return false
+}
+
 # Executes the "fossil http" command.  The entire content of the HTTP request
 # is read from the data file name, with [subst] being performed on it prior to
 # submission.  Temporary input and output files are created and deleted.  The
@@ -658,6 +979,11 @@ proc normalize_result {} {
   return [string map [list \r\n \n] [string trim $::RESULT]]
 }
 
+# fixup the line-endings in the result to make it easier to compare.
+proc normalize_result_no_trim {} {
+  return [string map [list \r\n \n] $::RESULT]
+}
+
 # returns the first line of the normalized result.
 proc first_data_line {} {
   return [lindex [split [normalize_result] \n] 0]
@@ -690,7 +1016,7 @@ proc third_to_last_data_line {} {
 
 set tempPath [getTemporaryPath]
 
-if {$tcl_platform(platform) eq "windows"} {
+if {$is_windows} {
   set tempPath [string map [list \\ /] $tempPath]
 }
 
@@ -711,11 +1037,18 @@ if {[catch {
 please set TEMP variable in environment, error: $error"
 }
 
+
 protInit $fossilexe
 set ::tempKeepHome 1
 foreach testfile $argv {
   protOut "***** $testfile ******"
-  source $testdir/$testfile.test
+  if { [catch {source $testdir/$testfile.test} testerror testopts] } {
+    test test-framework-$testfile 0
+    protOut "!!!!! $testfile: $testerror"
+    protOutDict $testopts"
+  } else {
+    test test-framework-$testfile 1
+  }
   protOut "***** End of $testfile: [llength $bad_test] errors so far ******"
 }
 unset ::tempKeepHome; delete_temporary_home
